@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2017-2020 Norbert Schlia (nschlia@oblivion-software.de)
+ * Copyright (C) 2017-2021 Norbert Schlia (nschlia@oblivion-software.de)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -28,7 +28,7 @@
  * @ingroup ffmpegfs
  *
  * @author Norbert Schlia (nschlia@oblivion-software.de)
- * @copyright Copyright (C) 2017-2020 Norbert Schlia (nschlia@oblivion-software.de)
+ * @copyright Copyright (C) 2017-2021 Norbert Schlia (nschlia@oblivion-software.de)
  */
 
 #include "blurayio.h"
@@ -59,7 +59,7 @@ BlurayIO::BlurayIO()
 
 BlurayIO::~BlurayIO()
 {
-
+    _close();
 }
 
 VIRTUALTYPE BlurayIO::type() const
@@ -147,7 +147,7 @@ int BlurayIO::open(LPVIRTUALFILE virtualfile)
 
     if (chapter_end > 0 && !m_full_title)
     {
-        m_end_pos = bd_chapter_pos(m_bd, chapter_end);
+        m_end_pos = bd_chapter_pos(m_bd, chapter_end) - 1;
     }
     else
     {
@@ -290,9 +290,9 @@ int BlurayIO::seek(int64_t offset, int whence)
         return (EOF);
     }
 
-    int64_t found_pos = bd_seek(m_bd, static_cast<uint64_t>(seek_pos));
-    m_cur_pos = static_cast<int64_t>(bd_tell(m_bd));
-    return (found_pos == seek_pos ? 0 : -1);
+    m_cur_pos = bd_seek(m_bd, static_cast<uint64_t>(seek_pos));
+
+    return (m_cur_pos == seek_pos ? 0 : -1);
 }
 
 bool BlurayIO::eof() const
@@ -302,7 +302,17 @@ bool BlurayIO::eof() const
 
 void BlurayIO::close()
 {
-    bd_close(m_bd);
+    _close();
+}
+
+void BlurayIO::_close()
+{
+    BLURAY * bd = m_bd;
+    if (bd != nullptr)
+    {
+        m_bd = nullptr;
+        bd_close(bd);
+    }
 }
 
 #endif // USE_LIBBLURAY
