@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2017-2020 Norbert Schlia (nschlia@oblivion-software.de)
+ * Copyright (C) 2017-2021 Norbert Schlia (nschlia@oblivion-software.de)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -28,7 +28,7 @@
  * @ingroup ffmpegfs
  *
  * @author Norbert Schlia (nschlia@oblivion-software.de)
- * @copyright Copyright (C) 2017-2020 Norbert Schlia (nschlia@oblivion-software.de)
+ * @copyright Copyright (C) 2017-2021 Norbert Schlia (nschlia@oblivion-software.de)
  */
 
 #include "ffmpegfs.h"
@@ -38,6 +38,7 @@
 #include "logging.h"
 
 #include "libbluray/bluray.h"
+#include "libbluray/bluray-version.h"
 
 extern "C" {
 #include <libavutil/rational.h>
@@ -157,6 +158,25 @@ static void stream_info(const std::string & path, BLURAY_STREAM_INFO *ss, int *c
             *height = 576;
             *interleaved = 0;
             break;
+        }
+            // Added with libluray change 14aa7e9c0 (hpi1 2017-08-28 09:50:43 +0300)
+            // Available since version 1.1.0
+#if (BLURAY_VERSION_MAJOR > 1 || (BLURAY_VERSION_MAJOR == 1 && BLURAY_VERSION_MINOR >= 1))
+        case BLURAY_VIDEO_FORMAT_2160P: // UHD
+        {
+            *width = 3840;
+            *height = 2160;
+            *interleaved = 0;
+            break;
+        }
+#endif
+        default:
+        {
+            Logging::error(path, "Unknown video format %1. Assuming 1920x1080P - may be totally wrong.", ss->format);
+            *width = 1920;
+            *height = 1080;
+            *interleaved = 0;
+            break;
         }
         }
 
@@ -192,6 +212,12 @@ static void stream_info(const std::string & path, BLURAY_STREAM_INFO *ss, int *c
             *framerate = av_make_q(60000, 1001);
             break;
         }
+        default:
+        {
+            Logging::error(path, "Unknown video frame rate %1. Assuming 25 fps - may be totally wrong.", ss->rate);
+            *framerate = av_make_q(25000, 1000);
+            break;
+        }
         }
 
         break;
@@ -218,7 +244,7 @@ static void stream_info(const std::string & path, BLURAY_STREAM_INFO *ss, int *c
         }
         case BLURAY_AUDIO_FORMAT_STEREO:
         {
-            *channels = 2;  // Multi Channel
+            *channels = 2;  // Stereo
             break;
         }
         case BLURAY_AUDIO_FORMAT_MULTI_CHAN:
@@ -231,6 +257,12 @@ static void stream_info(const std::string & path, BLURAY_STREAM_INFO *ss, int *c
             *channels = 2;  // Stereo ac3/dts
             break;
         }
+        default:
+        {
+            Logging::error(path, "Unknown number of audio channels %1. Assuming 2 channel/stereo - may be totally wrong.", ss->format);
+            *channels = 2;  // Stereo
+            break;
+        }
         }
 
         switch (ss->rate)
@@ -264,6 +296,12 @@ static void stream_info(const std::string & path, BLURAY_STREAM_INFO *ss, int *c
             // *sample_rate = "48/96 Khz";
             break;
         }
+        default:
+        {
+            Logging::error(path, "Unknown audio sample rate %1. Assuming 48 kHz - may be totally wrong.", ss->rate);
+            *sample_rate = 48000;
+            break;
+        }
         }
 
         *audio = 1;
@@ -291,7 +329,10 @@ static void stream_info(const std::string & path, BLURAY_STREAM_INFO *ss, int *c
 
 /**
  * @brief Find best match audio stream.
- * @todo is 0 really OK or shall we better parse?
+ * @todo Returning 0 is not necessarily the best match. Probably better to parse.
+ * For "The life of Brian", e.g., this is the Hungarian audio track. No big deal,
+ * though, I can recite mostly all dialogs in English and German (and Latin :),
+ * but should be fixed anyway.
  * @return Returns index of best match stream.
  */
 static int parse_find_best_audio_stream()
@@ -301,7 +342,8 @@ static int parse_find_best_audio_stream()
 
 /**
  * @brief Find best match video stream.
- * @todo is 0 really OK or shall we better parse?
+ * @todo Returning 0 is not necessarily the best match. Probably better to parse.
+ * Most DVDs contain only one video track anyway, so this does not hurt at the moment.
  * @return Returns index of best match stream.
  */
 static int parse_find_best_video_stream()
@@ -313,9 +355,9 @@ static int parse_find_best_video_stream()
  * @brief Create a virtual file entry of a bluray chapter or title.
  * @param[in] bd - Bluray disk clip info.
  * @param[in] ti - Bluray disk title info.
- * @param[in] path - path to check.
+ * @param[in] path - Path to check.
  * @param[in] statbuf - File status structure of original file.
- * @param[in, out] buf - the buffer passed to the readdir() operation.
+ * @param[in, out] buf - The buffer passed to the readdir() operation.
  * @param[in, out] filler - Function to add an entry in a readdir() operation (see https://libfuse.github.io/doxygen/fuse_8h.html#a7dd132de66a5cc2add2a4eff5d435660)
  * @param[in] is_main_title - true if title_idx is the main title
  * @param[in] full_title - If true, create virtual file of all title. If false, include single chapter only.
@@ -329,7 +371,6 @@ static bool create_bluray_virtualfile(BLURAY *bd, const BLURAY_TITLE_INFO* ti, c
     BLURAY_CLIP_INFO     *clip = &ti->clips[0];
     BLURAY_TITLE_CHAPTER *chapter = &ti->chapters[chapter_idx];
     char title_buf[PATH_MAX + 1];
-    struct stat stbuf;
     int64_t duration;
 
     if (full_title)
@@ -338,15 +379,15 @@ static bool create_bluray_virtualfile(BLURAY *bd, const BLURAY_TITLE_INFO* ti, c
 
         if (duration < AV_TIME_BASE)
         {
-            Logging::debug(path, "Title %1: skipping empty title", title_idx + 1);
+            Logging::trace(path, "Title %1: skipping empty title", title_idx + 1);
             return true;
         }
 
         snprintf(title_buf, sizeof(title_buf) - 1, "%02d. Title [%s]%s.%s",
-                title_idx + 1,
-                replace_all(format_duration(duration), ":", "-").c_str(),
-                is_main_title ? "+" : "",
-                params.m_format[0].fileext().c_str()); // can safely assume this is a video format
+                 title_idx + 1,
+                 replace_all(format_duration(duration), ":", "-").c_str(),
+                 is_main_title ? "+" : "",
+                 params.m_format[0].fileext().c_str()); // can safely assume this is a video format
     }
     else
     {
@@ -354,65 +395,41 @@ static bool create_bluray_virtualfile(BLURAY *bd, const BLURAY_TITLE_INFO* ti, c
 
         if (duration < AV_TIME_BASE)
         {
-            Logging::debug(path, "Title %1 Chapter %2: skipping empty chapter", title_idx + 1, chapter_idx + 1);
+            Logging::trace(path, "Title %1 Chapter %2: skipping empty chapter", title_idx + 1, chapter_idx + 1);
             return true;
         }
 
         snprintf(title_buf, sizeof(title_buf) - 1, "%02d. Chapter %03d [%s]%s.%s",
-                title_idx + 1,
-                chapter_idx + 1,
-                replace_all(format_duration(duration), ":", "-").c_str(),
-                is_main_title ? "+" : "",
-                params.m_format[0].fileext().c_str()); // can safely assume this is a video format
+                 title_idx + 1,
+                 chapter_idx + 1,
+                 replace_all(format_duration(duration), ":", "-").c_str(),
+                 is_main_title ? "+" : "",
+                 params.m_format[0].fileext().c_str()); // can safely assume this is a video format
 
     }
 
     std::string filename(title_buf);
 
-    memcpy(&stbuf, statbuf, sizeof(struct stat));
-
-#if defined __x86_64__ || !defined __USE_FILE_OFFSET64
-    stbuf.st_size   = 0; //static_cast<__off_t>(size);
-#else
-    stbuf.st_size   = 0; //static_cast<__off64_t>(size);
-#endif
-    stbuf.st_blocks = (stbuf.st_size + 512 - 1) / 512;
-
-    if (buf != nullptr && filler(buf, title_buf, &stbuf, 0))
-    {
-        // break;
-    }
-
     LPVIRTUALFILE virtualfile = nullptr;
     if (!params.m_format[0].is_multiformat())
     {
-        virtualfile = insert_file(VIRTUALTYPE_BLURAY, path + filename, &stbuf);
+        virtualfile = insert_file(VIRTUALTYPE_BLURAY, path + filename, statbuf);
     }
     else
     {
-        std::string origpath(path + filename);
+        virtualfile = insert_dir(VIRTUALTYPE_BLURAY, path + filename, statbuf);
+    }
 
-        int flags = VIRTUALFLAG_DIRECTORY | VIRTUALFLAG_FILESET;
+    if (virtualfile == nullptr)
+    {
+        Logging::error(path, "Failed to create virtual path: %1", (path + filename).c_str());
+        errno = EIO;
+        return false;
+    }
 
-        // Change file to directory for the frame set
-        // Change file to virtual directory for the frame set. Keep permissions.
-        stbuf.st_mode  &= ~static_cast<mode_t>(S_IFREG | S_IFLNK);
-        stbuf.st_mode  |= S_IFDIR;
-        stbuf.st_nlink = 2;
-        stbuf.st_size  = stbuf.st_blksize;
-
-        append_sep(&origpath);
-
-        if (params.m_format[0].is_frameset())
-        {
-            flags |= VIRTUALFLAG_FRAME;
-        }
-        else if (params.m_format[0].is_hls())
-        {
-            flags |= VIRTUALFLAG_HLS;
-        }
-
-        virtualfile = insert_file(VIRTUALTYPE_BLURAY, origpath, &stbuf, flags);
+    if (add_fuse_entry(buf, filler, title_buf, &virtualfile->m_st, 0))
+    {
+        // break;
     }
 
     // Bluray is video format anyway
@@ -424,7 +441,7 @@ static bool create_bluray_virtualfile(BLURAY *bd, const BLURAY_TITLE_INFO* ti, c
     virtualfile->m_bluray.m_chapter_no  = chapter_idx + 1;
     virtualfile->m_bluray.m_angle_no    = 1;
 
-    if (!transcoder_cached_filesize(virtualfile, &stbuf))
+    if (!transcoder_cached_filesize(virtualfile, &virtualfile->m_st))
     {
         BITRATE video_bit_rate  = 29*1024*1024; // In case the real bitrate cannot be calculated later, assume 20 Mbit video bitrate
         BITRATE audio_bit_rate  = 256*1024;     // In case the real bitrate cannot be calculated later, assume 256 kBit audio bitrate
@@ -451,8 +468,11 @@ static bool create_bluray_virtualfile(BLURAY *bd, const BLURAY_TITLE_INFO* ti, c
 
         if (duration)
         {
-            /** @todo We actually calculate the overall Bluray bitrate here, including all audio streams, not just the video bitrate. This should
-             * be the video bitrate alone. We should also calculate the audio bitrate for the selected stream. */
+            /**
+             * @todo We actually calculate the overall Bluray bitrate here, including all audio
+             * streams, not just the video bitrate. This should be the video bitrate alone. We
+             * should also calculate the audio bitrate for the selected stream.
+            */
             video_bit_rate      = static_cast<BITRATE>(size * 8LL * AV_TIME_BASE / static_cast<uint64_t>(duration));   // calculate bitrate in bps
         }
 
@@ -475,7 +495,7 @@ static bool create_bluray_virtualfile(BLURAY *bd, const BLURAY_TITLE_INFO* ti, c
         transcoder_set_filesize(virtualfile, duration, audio_bit_rate, channels, sample_rate, video_bit_rate, width, height, interleaved, framerate);
 
         virtualfile->m_video_frame_count    = static_cast<uint32_t>(av_rescale_q(duration, av_get_time_base_q(), av_inv_q(framerate)));
-        virtualfile->m_predicted_size       = size;
+        virtualfile->m_predicted_size       = static_cast<size_t>(size);
     }
 
     return true;
@@ -483,9 +503,9 @@ static bool create_bluray_virtualfile(BLURAY *bd, const BLURAY_TITLE_INFO* ti, c
 
 /**
  * @brief Parse Bluray directory and get all Bluray titles and chapters as virtual files.
- * @param[in] path - path to check.
+ * @param[in] path - Path to check.
  * @param[in] statbuf - File status structure of original file.
- * @param[in, out] buf - the buffer passed to the readdir() operation.
+ * @param[in, out] buf - The buffer passed to the readdir() operation.
  * @param[in, out] filler - Function to add an entry in a readdir() operation (see https://libfuse.github.io/doxygen/fuse_8h.html#a7dd132de66a5cc2add2a4eff5d435660)
  * @return On success, returns number of chapters found. On error, returns -errno.
  */
@@ -564,6 +584,8 @@ int check_bluray(const std::string & _path, void *buf, fuse_fill_dir_t filler)
         {
             res = load_path(path, &stbuf, buf, filler);
         }
+
+        add_dotdot(buf, filler, &stbuf, 0);
     }
     return res;
 }
