@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2017-2020 Norbert Schlia (nschlia@oblivion-software.de)
+ * Copyright (C) 2017-2021 Norbert Schlia (nschlia@oblivion-software.de)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -26,7 +26,7 @@
  * @ingroup ffmpegfs
  *
  * @author Norbert Schlia (nschlia@oblivion-software.de)
- * @copyright Copyright (C) 2017-2020 Norbert Schlia (nschlia@oblivion-software.de)
+ * @copyright Copyright (C) 2017-2021 Norbert Schlia (nschlia@oblivion-software.de)
  */
 
 #ifndef BUFFER_H
@@ -424,7 +424,7 @@ private:
      * @param[in] filesize - Actual size of the cache file.
      * @return Returns true on success; false on error.
      */
-    bool                    unmap_file(const std::string & filename, int *fd, uint8_t **p, size_t *filesize, size_t *buffer_pos = nullptr) const;
+    bool                    unmap_file(const std::string & filename, int *fd, uint8_t **p, size_t *filesize) const;
 
     /**
      * @brief cacheinfo
