@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2017-2020 Norbert Schlia (nschlia@oblivion-software.de)
+ * Copyright (C) 2017-2021 Norbert Schlia (nschlia@oblivion-software.de)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -26,7 +26,7 @@
  * @ingroup ffmpegfs
  *
  * @author Norbert Schlia (nschlia@oblivion-software.de)
- * @copyright Copyright (C) 2017-2020 Norbert Schlia (nschlia@oblivion-software.de)
+ * @copyright Copyright (C) 2017-2021 Norbert Schlia (nschlia@oblivion-software.de)
  */
 
 #ifndef CACHE_H
@@ -39,14 +39,14 @@
 #include <map>
 #include <sqlite3.h>
 
-#define     DB_BASE_VERSION_MAJOR   1           /**< brief The oldest database version major (Release < 1.95) */
-#define     DB_BASE_VERSION_MINOR   0           /**< brief The oldest database version minor (Release < 1.95) */
+#define     DB_BASE_VERSION_MAJOR   1           /**< @brief The oldest database version major (Release < 1.95) */
+#define     DB_BASE_VERSION_MINOR   0           /**< @brief The oldest database version minor (Release < 1.95) */
 
-#define     DB_VERSION_MAJOR        1           /**< brief Current database version major */
-#define     DB_VERSION_MINOR        97          /**< brief Current database version minor */
+#define     DB_VERSION_MAJOR        1           /**< @brief Current database version major */
+#define     DB_VERSION_MINOR        97          /**< @brief Current database version minor */
 
-#define     DB_MIN_VERSION_MAJOR    1           /**< brief Required database version major (required 1.95) */
-#define     DB_MIN_VERSION_MINOR    97          /**< brief Required database version minor (required 1.95) */
+#define     DB_MIN_VERSION_MAJOR    1           /**< @brief Required database version major (required 1.95) */
+#define     DB_MIN_VERSION_MINOR    97          /**< @brief Required database version minor (required 1.95) */
 
 /**
   * @brief RESULTCODE of transcoding operation
@@ -100,8 +100,8 @@ class Cache_Entry;
  */
 class Cache
 {
-    typedef std::pair<std::string, std::string> cache_key_t;
-    typedef std::map<cache_key_t, Cache_Entry *> cache_t;
+    typedef std::pair<std::string, std::string> cache_key_t;    ///< Filenames and destination types
+    typedef std::map<cache_key_t, Cache_Entry *> cache_t;       ///< Map of cache entries
 public:
     /**
       * @brief Definition of sql table
@@ -131,7 +131,7 @@ public:
     /**
      * @brief Construct #Cache object.
      */
-    Cache();
+    explicit Cache();
     /**
      * @brief Destruct #Cache object.
      */
@@ -316,13 +316,13 @@ protected:
 
 private:
     static const
-    TABLE_DEF               m_table_cache_entry;
+    TABLE_DEF               m_table_cache_entry;            /**< @brief Definition and indexes of table "cache_entry" */
     static const
-    TABLE_COLUMNS           m_columns_cache_entry[];
+    TABLE_COLUMNS           m_columns_cache_entry[];        /**< @brief Columns of table "cache_entry" */
     static const
-    TABLE_DEF               m_table_version;
+    TABLE_DEF               m_table_version;                /**< @brief Definition and indexes of table "version" */
     static const
-    TABLE_COLUMNS           m_columns_version[];
+    TABLE_COLUMNS           m_columns_version[];            /**< @brief Columns of table "version" */
 
     std::recursive_mutex    m_mutex;                        /**< @brief Access mutex */
     std::string             m_cacheidx_file;                /**< @brief Name of SQLite cache index database */
