@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2017-2020 Norbert Schlia (nschlia@oblivion-software.de)
+ * Copyright (C) 2017-2021 Norbert Schlia (nschlia@oblivion-software.de)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -26,7 +26,7 @@
  * @ingroup ffmpegfs
  *
  * @author Norbert Schlia (nschlia@oblivion-software.de)
- * @copyright Copyright (C) 2017-2020 Norbert Schlia (nschlia@oblivion-software.de)
+ * @copyright Copyright (C) 2017-2021 Norbert Schlia (nschlia@oblivion-software.de)
  */
 
 #include "cache_entry.h"
@@ -81,7 +81,7 @@ bool Cache_Entry::destroy()
 {
     //Logging::info(filename(), "TEST: Cache_Entry deleted.");
 
-    delete this;    /** @todo implement delete later mechanism */
+    delete this;    /** @todo Implement delete later mechanism */
 
     return true;    /** @todo Return true when deleted, false if kept for delete later */
 }
