@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2017-2020 Norbert Schlia (nschlia@oblivion-software.de)
+ * Copyright (C) 2017-2021 Norbert Schlia (nschlia@oblivion-software.de)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -28,7 +28,7 @@
  * @ingroup ffmpegfs
  *
  * @author Norbert Schlia (nschlia@oblivion-software.de)
- * @copyright Copyright (C) 2017-2020 Norbert Schlia (nschlia@oblivion-software.de)
+ * @copyright Copyright (C) 2017-2021 Norbert Schlia (nschlia@oblivion-software.de)
  */
 
 #include "dvdio.h"
@@ -72,6 +72,7 @@ DvdIO::DvdIO()
 
 DvdIO::~DvdIO()
 {
+    _close();
 }
 
 VIRTUALTYPE DvdIO::type() const
@@ -115,7 +116,7 @@ int DvdIO::open(LPVIRTUALFILE virtualfile)
 
     // Open the disc.
     m_dvd = DVDOpen(path().c_str());
-    if (!m_dvd)
+    if (m_dvd == nullptr)
     {
         Logging::error(path(), "Couldn't open DVD.");
         return EINVAL;
@@ -123,7 +124,7 @@ int DvdIO::open(LPVIRTUALFILE virtualfile)
 
     // Load the video manager to find out the information about the titles on this disc.
     m_vmg_file = ifoOpen(m_dvd, 0);
-    if (!m_vmg_file)
+    if (m_vmg_file == nullptr)
     {
         Logging::error(path(), "Can't open VMG info.");
         DVDClose(m_dvd);
@@ -166,7 +167,7 @@ int DvdIO::open(LPVIRTUALFILE virtualfile)
 
     // Load the VTS information for the title set our title is in.
     m_vts_file = ifoOpen(m_dvd, tt_srpt->title[m_title_idx].title_set_nr);
-    if (!m_vts_file)
+    if (m_vts_file == nullptr)
     {
         Logging::error(path(), "Can't open the title %1 info file.", tt_srpt->title[m_title_idx].title_set_nr);
         ifoClose(m_vmg_file);
@@ -193,7 +194,7 @@ int DvdIO::open(LPVIRTUALFILE virtualfile)
 
     // We've got enough info, time to open the title set data.
     m_dvd_title = DVDOpenFile(m_dvd, tt_srpt->title[m_title_idx].title_set_nr, DVD_READ_TITLE_VOBS);
-    if (!m_dvd_title)
+    if (m_dvd_title == nullptr)
     {
         Logging::error(path(), "Can't open title VOBS (VTS_%<%02d>1_1.VOB).", tt_srpt->title[m_title_idx].title_set_nr);
         ifoClose(m_vts_file);
@@ -793,27 +794,35 @@ bool DvdIO::eof() const
 
 void DvdIO::close()
 {
-    if (m_vts_file != nullptr)
-    {
-        ifoClose(m_vts_file);
-        m_vts_file = nullptr;
-    }
-    if (m_vmg_file != nullptr)
-    {
-        ifoClose(m_vmg_file);
-        m_vmg_file = nullptr;
-    }
-    if (m_dvd_title != nullptr)
-    {
-        DVDCloseFile(m_dvd_title);
-        m_dvd_title = nullptr;
-    }
-    if (m_dvd != nullptr)
-    {
-        DVDClose(m_dvd);
-        m_dvd = nullptr;
-    }
+    _close();
+}
 
+void DvdIO::_close()
+{
+    ifo_handle_t *  vts_file = m_vts_file;
+    if (vts_file != nullptr)
+    {
+        m_vts_file = nullptr;
+        ifoClose(vts_file);
+    }
+    ifo_handle_t *  vmg_file  = m_vmg_file;
+    if (vmg_file != nullptr)
+    {
+        m_vmg_file = nullptr;
+        ifoClose(vmg_file);
+    }
+    dvd_file_t *    dvd_title = m_dvd_title;
+    if (dvd_title != nullptr)
+    {
+        m_dvd_title = nullptr;
+        DVDCloseFile(dvd_title);
+    }
+    dvd_reader_t *  dvd = m_dvd;
+    if (dvd != nullptr)
+    {
+        m_dvd = nullptr;
+        DVDClose(dvd);
+    }
 }
 
 // Code nicked from Handbrake (https://github.com/HandBrake/HandBrake/blob/master/libhb/dvd.c)
