@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2018-2020 by Norbert Schlia (nschlia@oblivion-software.de)
+ * Copyright (C) 2018-2021 by Norbert Schlia (nschlia@oblivion-software.de)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -28,7 +28,7 @@
  * @ingroup ffmpegfs
  *
  * @author Norbert Schlia (nschlia@oblivion-software.de)
- * @copyright Copyright (C) 2018-2020 Norbert Schlia (nschlia@oblivion-software.de)
+ * @copyright Copyright (C) 2018-2021 Norbert Schlia (nschlia@oblivion-software.de)
  */
 
 #ifndef DVDIO_H
@@ -81,13 +81,15 @@ public:
      */
     virtual size_t  bufsize() const;
 
-    /** @brief Open a virtual file
+    /**
+     * @brief Open a virtual file
      * @param[in] virtualfile - LPCVIRTUALFILE of file to open
      * @return Upon successful completion, #open() returns 0. @n
      * On error, an nonzero value is returned and errno is set to indicate the error.
      */
     virtual int     open(LPVIRTUALFILE virtualfile);
-    /** @brief Read data from file
+    /**
+     * @brief Read data from file
      * @param[out] data - buffer to store read bytes in. Must be large enough to hold up to size bytes.
      * Special case: If set to nullptr as many bytes as possible are "read" (and discarded). Can be used
      * to determine the net file size of the virtual stream.
@@ -103,7 +105,8 @@ public:
      * @return errno value of last error.
      */
     virtual int     error() const;
-    /** @brief Get the duration of the file, in AV_TIME_BASE fractional seconds.
+    /**
+     * @brief Get the duration of the file, in AV_TIME_BASE fractional seconds.
      * @return Returns the duration of the file, in AV_TIME_BASE fractional seconds.
      */
     virtual int64_t duration() const;
@@ -117,7 +120,8 @@ public:
      * @return Gets the current read position.
      */
     virtual size_t  tell() const;
-    /** @brief Seek to position in file
+    /**
+     * @brief Seek to position in file
      *
      * Repositions the offset of the open file to the argument offset according to the directive whence.
      *
@@ -142,6 +146,12 @@ public:
     virtual void    close();
 
 private:
+    /**
+     * @brief Close virtual file.
+     * Non-virtual version to be safely called from constructor/destructor
+     */
+    void            _close();
+
     /**
      * @brief Do a rough check if this is really a navigation packet.
      * @param[in] buffer - Buffer with data.
