@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2017-2020 Norbert Schlia (nschlia@oblivion-software.de)
+ * Copyright (C) 2017-2021 Norbert Schlia (nschlia@oblivion-software.de)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -26,38 +26,27 @@
  * @ingroup ffmpegfs
  *
  * @author Norbert Schlia (nschlia@oblivion-software.de)
- * @copyright Copyright (C) 2017-2020 Norbert Schlia (nschlia@oblivion-software.de)
+ * @copyright Copyright (C) 2017-2021 Norbert Schlia (nschlia@oblivion-software.de)
  */
 
 #include "ffmpeg_base.h"
 #include "transcode.h"
 #include "logging.h"
 
+#ifdef __cplusplus
+extern "C" {
+#endif
 // Disable annoying warnings outside our code
 #pragma GCC diagnostic push
 #pragma GCC diagnostic ignored "-Wconversion"
 #pragma GCC diagnostic ignored "-Wsign-conversion"
-#ifdef __GNUC__
-#  include <features.h>
-#  if __GNUC_PREREQ(5,0) || defined(__clang__)
-// GCC >= 5.0
-#     pragma GCC diagnostic ignored "-Wfloat-conversion"
-#  elif __GNUC_PREREQ(4,8)
-// GCC >= 4.8
-#  else
-#     error("GCC < 4.8 not supported");
-#  endif
-#endif
-#ifdef __cplusplus
-extern "C" {
-#endif
 #include <libavutil/opt.h>
 #include <libavutil/mathematics.h>
 #include <libavutil/pixdesc.h>
+#pragma GCC diagnostic pop
 #ifdef __cplusplus
 }
 #endif
-#pragma GCC diagnostic pop
 
 FFmpeg_Base::FFmpeg_Base()
     : m_virtualfile(nullptr)
@@ -68,89 +57,6 @@ FFmpeg_Base::~FFmpeg_Base()
 {
 }
 
-int FFmpeg_Base::open_bestmatch_codec_context(AVCodecContext **avctx, int *stream_idx, AVFormatContext *fmt_ctx, AVMediaType type, const char *filename) const
-{
-    int ret;
-
-    ret = av_find_best_stream(fmt_ctx, type, INVALID_STREAM, INVALID_STREAM, nullptr, 0);
-    if (ret < 0)
-    {
-        if (ret != AVERROR_STREAM_NOT_FOUND)    // Not an error
-        {
-            Logging::error(filename, "Could not find %1 stream in input file (error '%2').", get_media_type_string(type), ffmpeg_geterror(ret).c_str());
-        }
-        return ret;
-    }
-
-    *stream_idx = ret;
-
-    return open_codec_context(avctx, *stream_idx, fmt_ctx, type, filename);
-}
-
-int FFmpeg_Base::open_codec_context(AVCodecContext **avctx, int stream_idx, AVFormatContext *fmt_ctx, AVMediaType type, const char *filename) const
-{
-    AVCodecContext *dec_ctx = nullptr;
-    AVCodec *dec = nullptr;
-    AVDictionary *opts = nullptr;
-    AVStream *input_stream;
-    AVCodecID codec_id = AV_CODEC_ID_NONE;
-    int ret;
-
-    input_stream = fmt_ctx->streams[stream_idx];
-
-    // Init the decoders, with or without reference counting
-    // av_dict_set_with_check(&opts, "refcounted_frames", refcount ? "1" : "0", 0);
-
-#if LAVF_DEP_AVSTREAM_CODEC
-    // allocate a new decoding context
-    dec_ctx = avcodec_alloc_context3(nullptr);
-    if (dec_ctx == nullptr)
-    {
-        Logging::error(filename, "Could not allocate a decoding context.");
-        return AVERROR(ENOMEM);
-    }
-
-    // initialise the stream parameters with demuxer information
-    ret = avcodec_parameters_to_context(dec_ctx, input_stream->codecpar);
-    if (ret < 0)
-    {
-        return ret;
-    }
-
-    codec_id = input_stream->codecpar->codec_id;
-#else
-    dec_ctx = input_stream->codec;
-
-    codec_id = dec_ctx->codec_id;
-#endif
-
-    // Find a decoder for the stream.
-    dec = avcodec_find_decoder(codec_id);
-    if (dec == nullptr)
-    {
-        Logging::error(filename, "Failed to find %1 input codec.", get_media_type_string(type));
-        return AVERROR(EINVAL);
-    }
-
-    dec_ctx->codec_id = dec->id;
-
-    ret = avcodec_open2(dec_ctx, dec, &opts);
-
-    av_dict_free(&opts);
-
-    if (ret < 0)
-    {
-        Logging::error(filename, "Failed to open %1 input codec for stream #%1 (error '%2').", get_media_type_string(type), input_stream->index, ffmpeg_geterror(ret).c_str());
-        return ret;
-    }
-
-    Logging::debug(filename, "Opened input codec for stream #%1: %2", input_stream->index, get_codec_name(codec_id, true));
-
-    *avctx = dec_ctx;
-
-    return 0;
-}
-
 void FFmpeg_Base::init_packet(AVPacket *pkt) const
 {
     av_init_packet(pkt);
@@ -161,7 +67,7 @@ void FFmpeg_Base::init_packet(AVPacket *pkt) const
 
 int FFmpeg_Base::init_frame(AVFrame **frame, const char *filename) const
 {
-    *frame = ::av_frame_alloc();
+    *frame = av_frame_alloc();
     if (*frame == nullptr)
     {
         Logging::error(filename, "Could not allocate frame.");
@@ -274,7 +180,7 @@ void FFmpeg_Base::video_stream_setup(AVCodecContext *output_codec_ctx, AVStream*
     output_codec_ctx->gop_size                  = 12;   // emit one intra frame every twelve frames at most
 }
 
-int FFmpeg_Base::av_dict_set_with_check(AVDictionary **pm, const char *key, const char *value, int flags, const char * filename) const
+int FFmpeg_Base::dict_set_with_check(AVDictionary **pm, const char *key, const char *value, int flags, const char * filename) const
 {
     int ret = av_dict_set(pm, key, value, flags);
 
@@ -286,7 +192,7 @@ int FFmpeg_Base::av_dict_set_with_check(AVDictionary **pm, const char *key, cons
     return ret;
 }
 
-int FFmpeg_Base::av_opt_set_with_check(void *obj, const char *key, const char *value, int flags, const char * filename) const
+int FFmpeg_Base::opt_set_with_check(void *obj, const char *key, const char *value, int flags, const char * filename) const
 {
     int ret = av_opt_set(obj, key, value, flags);
 
@@ -332,21 +238,21 @@ void FFmpeg_Base::audio_info(bool out_file, const AVFormatContext *format_ctx, c
                   format_duration(duration).c_str());
 }
 
-std::string FFmpeg_Base::get_pix_fmt_name(enum AVPixelFormat pix_fmt) const
+std::string FFmpeg_Base::get_pix_fmt_name(enum AVPixelFormat pix_fmt)
 {
-    const char *fmt_name = ::av_get_pix_fmt_name(pix_fmt);
+    const char *fmt_name = av_get_pix_fmt_name(pix_fmt);
     return (fmt_name != nullptr ? fmt_name : "none");
 }
 
-std::string FFmpeg_Base::get_sample_fmt_name(AVSampleFormat sample_fmt) const
+std::string FFmpeg_Base::get_sample_fmt_name(AVSampleFormat sample_fmt)
 {
     return av_get_sample_fmt_name(sample_fmt);
 }
 
-std::string FFmpeg_Base::get_channel_layout_name(int nb_channels, uint64_t channel_layout) const
+std::string FFmpeg_Base::get_channel_layout_name(int nb_channels, uint64_t channel_layout)
 {
     char buffer[1024];
-    av_get_channel_layout_string(buffer, sizeof(buffer), nb_channels, channel_layout);
+    av_get_channel_layout_string(buffer, sizeof(buffer) - 1, nb_channels, channel_layout);
     return buffer;
 }
 
