@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2017-2020 Norbert Schlia (nschlia@oblivion-software.de)
+ * Copyright (C) 2017-2021 Norbert Schlia (nschlia@oblivion-software.de)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -26,39 +26,27 @@
  * @ingroup ffmpegfs
  *
  * @author Norbert Schlia (nschlia@oblivion-software.de)
- * @copyright Copyright (C) 2017-2020 Norbert Schlia (nschlia@oblivion-software.de)
+ * @copyright Copyright (C) 2017-2021 Norbert Schlia (nschlia@oblivion-software.de)
  */
 
 #include "ffmpeg_utils.h"
 #include "id3v1tag.h"
 #include "ffmpegfs.h"
 
+#include <iostream>
 #include <libgen.h>
 #include <unistd.h>
 #include <algorithm>
-#include <regex.h>
 #include <wordexp.h>
 #include <memory>
-#include <regex>
 
+#ifdef __cplusplus
+extern "C" {
+#endif
 // Disable annoying warnings outside our code
 #pragma GCC diagnostic push
 #pragma GCC diagnostic ignored "-Wconversion"
 #pragma GCC diagnostic ignored "-Wsign-conversion"
-#ifdef __GNUC__
-#  include <features.h>
-#  if __GNUC_PREREQ(5,0) || defined(__clang__)
-// GCC >= 5.0
-#     pragma GCC diagnostic ignored "-Wfloat-conversion"
-#  elif __GNUC_PREREQ(4,8)
-// GCC >= 4.8
-#  else
-#     error("GCC < 4.8 not supported");
-#  endif
-#endif
-#ifdef __cplusplus
-extern "C" {
-#endif
 #include <libswscale/swscale.h>
 #if LAVR_DEPRECATE
 #include <libswresample/swresample.h>
@@ -66,10 +54,10 @@ extern "C" {
 #include <libavresample/avresample.h>
 #endif
 #include "libavutil/ffversion.h"
+#pragma GCC diagnostic pop
 #ifdef __cplusplus
 }
 #endif
-#pragma GCC diagnostic pop
 
 static int is_device(__attribute__((unused)) const AVClass *avclass);
 static std::string ffmpeg_libinfo(bool lib_exists, __attribute__((unused)) unsigned int version, __attribute__((unused)) const char *cfg, int version_minor, int version_major, int version_micro, const char * libname);
@@ -556,7 +544,7 @@ static int is_device(__attribute__((unused)) const AVClass *avclass)
     //return AV_IS_INPUT_DEVICE(avclass->category) || AV_IS_OUTPUT_DEVICE(avclass->category);
 }
 
-int show_formats_devices(int device_only)
+int show_caps(int device_only)
 {
     const AVInputFormat *ifmt  = nullptr;
     const AVOutputFormat *ofmt = nullptr;
@@ -679,7 +667,7 @@ int mktree(const std::string & path, mode_t mode)
         return ENOMEM;
     }
 
-    char dir[PATH_MAX] = "\0";
+    std::string dir;
     char *p = strtok (buffer, "/");
     int status = 0;
 
@@ -687,12 +675,12 @@ int mktree(const std::string & path, mode_t mode)
     {
         int newstat;
 
-        strcat(dir, "/");
-        strcat(dir, p);
+        dir += "/";
+        dir += p;
 
         errno = 0;
 
-        newstat = mkdir(dir, mode);
+        newstat = mkdir(dir.c_str(), mode);
 
         if (!status && newstat && errno != EEXIST)
         {
@@ -1112,7 +1100,7 @@ std::string replace_all(std::string str, const std::string& from, const std::str
 
 int strcasecmp(const std::string & s1, const std::string & s2)
 {
-    return strcasecmp(s1.c_str(), s2.c_str());
+    return ::strcasecmp(s1.c_str(), s2.c_str());
 }
 
 template<typename ... Args>
@@ -1124,21 +1112,24 @@ std::string string_format(const std::string& format, Args ... args)
     return std::string(buf.get(), buf.get() + size - 1); // We don't want the '\0' inside
 }
 
-int compare(const std::string & value, const std::string & pattern)
+int reg_compare(const std::string & value, const std::string & pattern, std::regex::flag_type flag)
 {
-    regex_t regex;
     int reti;
 
-    reti = regcomp(&regex, pattern.c_str(), REG_EXTENDED | REG_ICASE);
-    if (reti)
+    try
     {
-        std::fprintf(stderr, "Could not compile regex\n");
-        return -1;
+        std::regex rgx(pattern, flag);
+
+        reti = (std::regex_search(value, rgx) == true) ? 0 : 1;
     }
+    catch(const std::regex_error& e)
+    {
+        std::cerr << "regex_error caught: " << e.what() << '\n';
+        if(e.code() == std::regex_constants::error_brack)
+            std::cerr << "The code was error_brack\n";
 
-    reti = regexec(&regex, value.c_str(), 0, nullptr, 0);
-
-    regfree(&regex);
+        reti = -1;
+    }
 
     return reti;
 }
@@ -1236,7 +1227,7 @@ std::vector<std::string> split(const std::string& input, const std::string & reg
 {
     // passing -1 as the submatch index parameter performs splitting
     std::regex re(regex);
-    std::sregex_token_iterator first{input.begin(), input.end(), re, -1},
+    std::sregex_token_iterator first{input.cbegin(), input.cend(), re, -1},
     last;
     return {first, last};
 }
@@ -1264,9 +1255,33 @@ std::string sanitise_filepath(const std::string & filepath)
     return sanitise_filepath(&buffer);
 }
 
-bool is_album_art(AVCodecID codec_id)
+bool is_album_art(AVCodecID codec_id, const AVRational * frame_rate)
 {
-    return (codec_id == AV_CODEC_ID_MJPEG || codec_id == AV_CODEC_ID_PNG || codec_id == AV_CODEC_ID_BMP);
+    if (codec_id == AV_CODEC_ID_PNG || codec_id == AV_CODEC_ID_BMP)
+    {
+        // PNG or BMP: must be an album art stream
+        return true;
+    }
+
+    if (codec_id != AV_CODEC_ID_MJPEG)
+    {
+        // Anything else than MJPEG is never an album art stream
+        return false;
+    }
+
+    if (frame_rate != nullptr && frame_rate->den)
+    {
+        double dbFrameRate = static_cast<double>(frame_rate->num) / frame_rate->den;
+
+        // If frame rate is < 300 fps this most likely is a video
+        if (dbFrameRate < 300)
+        {
+            // This is a video
+            return false;
+        }
+    }
+
+    return true;
 }
 
 bool nocasecompare(const std::string & lhs, const std::string &rhs)
@@ -1318,3 +1333,28 @@ bool file_exists(const std::string & filename)
 {
     return (access(filename.c_str(), F_OK) != -1);
 }
+
+void make_upper(std::string * input)
+{
+    std::for_each(std::begin(*input), std::end(*input), [](char& c) {
+        c = static_cast<char>(std::toupper(static_cast<unsigned char>(c)));
+    });
+}
+
+void make_lower(std::string * input)
+{
+    std::for_each(std::begin(*input), std::end(*input), [](char& c) {
+        c = static_cast<char>(std::tolower(static_cast<unsigned char>(c)));
+    });
+}
+
+void stat_set_size(struct stat *st, size_t size)
+{
+#if defined __x86_64__ || !defined __USE_FILE_OFFSET64
+    st->st_size       = static_cast<__off_t>(size);
+#else
+    st->st_size       = static_cast<__off64_t>(size);
+#endif
+    st->st_blocks     = (st->st_size + 512 - 1) / 512;
+}
+
