@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2017-2020 by Norbert Schlia (nschlia@oblivion-software.de)
+ * Copyright (C) 2017-2021 by Norbert Schlia (nschlia@oblivion-software.de)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -26,7 +26,7 @@
  * @ingroup ffmpegfs
  *
  * @author Norbert Schlia (nschlia@oblivion-software.de)
- * @copyright Copyright (C) 2017-2020 Norbert Schlia (nschlia@oblivion-software.de)
+ * @copyright Copyright (C) 2017-2021 Norbert Schlia (nschlia@oblivion-software.de)
  */
 
 #ifndef FFMPEG_UTILS_H
@@ -38,12 +38,19 @@
 #include "config.h"
 #endif
 
-#define __STDC_FORMAT_MACROS                    /**< @brief Force PRId64 defines */
+#if !defined(EXTRA_VERSION)
+#define FFMPEFS_VERSION     PACKAGE_VERSION                     /**< @brief FFmpegfs version number */
+#else
+#define FFMPEFS_VERSION     PACKAGE_VERSION "-" EXTRA_VERSION   /**< @brief FFmpegfs version number */
+#endif
+
+#define __STDC_FORMAT_MACROS                                    /**< @brief Force PRId64 defines */
 
 #include "ffmpeg_compat.h"
 
 #include <string>
 #include <vector>
+#include <regex>
 
 #if !defined(USE_LIBSWRESAMPLE) && !defined(USE_LIBAVRESAMPLE)
 #error "Must have either libswresample (preferred choice for FFMpeg) or libavresample (with libav)."
@@ -59,29 +66,18 @@
 #include <linux/limits.h>
 #endif
 
+#ifdef __cplusplus
+extern "C" {
+#endif
 // Disable annoying warnings outside our code
 #pragma GCC diagnostic push
 #pragma GCC diagnostic ignored "-Wconversion"
 #pragma GCC diagnostic ignored "-Wsign-conversion"
-#ifdef __GNUC__
-#  include <features.h>
-#  if __GNUC_PREREQ(5,0) || defined(__clang__)
-// GCC >= 5.0
-#     pragma GCC diagnostic ignored "-Wfloat-conversion"
-#  elif __GNUC_PREREQ(4,8)
-// GCC >= 4.8
-#  else
-#     error("GCC < 4.8 not supported");
-#  endif
-#endif
-#ifdef __cplusplus
-extern "C" {
-#endif
 #include <libavformat/avformat.h>
+#pragma GCC diagnostic pop
 #ifdef __cplusplus
 }
 #endif
-#pragma GCC diagnostic pop
 
 #ifndef LIBAVUTIL_VERSION_MICRO
 #error "LIBAVUTIL_VERSION_MICRO not defined. Missing include header?"
@@ -517,6 +513,16 @@ template<typename ... Args> std::string string_format(const std::string& format,
  * @return Returns same as strcasecmp() for char *.
  */
 int                 strcasecmp(const std::string & s1, const std::string & s2);
+/**
+ * @brief Convert string to upper case
+ * @param[inout] input String to convert
+ */
+void                make_upper(std::string * input);
+/**
+ * @brief Convert string to lower case
+ * @param[inout] input String to convert
+ */
+void                make_lower(std::string * input);
 /**
  * @brief Get info about the FFmpeg libraries used.
  * @return std::tring with info about the linked FFmpeg libraries.
@@ -527,7 +533,7 @@ std::string         ffmpeg_libinfo();
  * @param[in] device_only - If true lists devices only.
  * @return On success returns 0; on error negative AVERROR.
  */
-int                 show_formats_devices(int device_only);
+int                 show_caps(int device_only);
 /**
  * @brief Safe way to get the codec name. Function never fails, will return "unknown" on error.
  * @param[in] codec_id - ID of codec
@@ -564,9 +570,10 @@ int                 print_stream_info(const AVStream* stream);
  * @brief Compare value with pattern.
  * @param[in] value - Value to check.
  * @param[in] pattern - Regexp pattern to match.
+ * @param[in] flag - On of the flag_type constants, see https://en.cppreference.com/w/cpp/regex/basic_regex for options. Mostly std::regex::icase is used to make matches case insensitive.
  * @return Returns 0 if pattern matches; 1 if not; -1 if pattern is no valid regex
  */
-int                 compare(const std::string &value, const std::string &pattern);
+int                 reg_compare(const std::string &value, const std::string &pattern, std::regex::flag_type flag = std::regex::ECMAScript);
 /**
  * @brief Expand path, e.g., expand ~/ to home directory.
  * @param[out] tgt - Expanded source path.
@@ -626,10 +633,14 @@ std::string         sanitise_filepath(std::string * filepath);
 
 /**
  * @brief Minimal check if codec is an album art.
+ * Requires frame_rate to decide whether this is a video stream if codec_id is
+ * not BMP or PNG (which means its undoubtedly an album art). For MJPEG this may
+ * also be a video stream if the frame rate is high enough.
  * @param[in] codec_id - ID of codec.
+ * @param[in] frame_rate - Video frame rate, if known.
  * @return Returns true if codec is for an image; false if not.
  */
-bool                is_album_art(AVCodecID codec_id);
+bool                is_album_art(AVCodecID codec_id, const AVRational *frame_rate = nullptr);
 
 /**
  * @brief nocasecompare to make std::string find operations case insensitive
@@ -686,4 +697,11 @@ std::string         make_filename(uint32_t file_no, const std::string &fileext);
  */
 bool                file_exists(const std::string & filename);
 
+/**
+ * @brief Properly fill in all size related members in stat struct
+ * @param st[inout] stat structure to update
+ * @param size[in] size value to copy
+ */
+void                stat_set_size(struct stat *st, size_t size);
+
 #endif
