@@ -1,7 +1,7 @@
 /*
  * Copyright (C) 2006-2008 David Collett
  * Copyright (C) 2008-2012 K. Henriksson
- * Copyright (C) 2017-2020 FFmpeg support by Norbert Schlia (nschlia@oblivion-software.de)
+ * Copyright (C) 2017-2021 FFmpeg support by Norbert Schlia (nschlia@oblivion-software.de)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -30,7 +30,7 @@
  * @author Norbert Schlia (nschlia@oblivion-software.de)
  * @copyright Copyright (C) 2006-2008 David Collett @n
  * Copyright (C) 2008-2013 K. Henriksson @n
- * Copyright (C) 2017-2020 FFmpeg support by Norbert Schlia (nschlia@oblivion-software.de)
+ * Copyright (C) 2017-2021 FFmpeg support by Norbert Schlia (nschlia@oblivion-software.de)
  */
 
 #include "ffmpegfs.h"
@@ -47,30 +47,19 @@
 #include <libbluray/bluray-version.h>
 #endif
 
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wconversion"
-#pragma GCC diagnostic ignored "-Wsign-conversion"
-#ifdef __GNUC__
-#  include <features.h>
-#  if __GNUC_PREREQ(5,0) || defined(__clang__)
-// GCC >= 5.0
-#     pragma GCC diagnostic ignored "-Wfloat-conversion"
-#  elif __GNUC_PREREQ(4,8)
-// GCC >= 4.8
-#  else
-#     error("GCC < 4.8 not supported");
-#  endif
-#endif
 #ifdef __cplusplus
 extern "C" {
 #endif
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wconversion"
+#pragma GCC diagnostic ignored "-Wsign-conversion"
 #include <libavcodec/avcodec.h>
 #include <libavformat/avformat.h>
 #include <libavfilter/avfilter.h>
+#pragma GCC diagnostic pop
 #ifdef __cplusplus
 }
 #endif
-#pragma GCC diagnostic pop
 
 #include "ffmpeg_utils.h"
 
@@ -81,7 +70,7 @@ FFMPEGFS_PARAMS::FFMPEGFS_PARAMS()
     , m_mountpath("")                           // required parameter
 
     , m_autocopy(AUTOCOPY_OFF)                  // default: off
-    , m_profile(PROFILE_DEFAULT)                   // default: no profile
+    , m_profile(PROFILE_DEFAULT)                // default: no profile
     , m_level(PRORESLEVEL_NONE)                 // default: no level
 
     // Format
@@ -122,7 +111,7 @@ FFMPEGFS_PARAMS::FFMPEGFS_PARAMS()
     , m_decoding_errors(0)                      // default: ignore errors
     , m_min_dvd_chapter_duration(1)             // default: 1 second
     , m_oldnamescheme(0)                        // default: new scheme
-    , m_win_smb_fix(0)                          // default: no fix
+    , m_win_smb_fix(1)                          // default: fix enabled
 {
 }
 
@@ -196,6 +185,7 @@ enum
 {
     KEY_HELP,
     KEY_VERSION,
+    KEY_FFMPEG_CAPS,
     KEY_KEEP_OPT,
     // Intelligent parameters
     KEY_DESTTYPE,
@@ -308,8 +298,8 @@ static struct fuse_opt ffmpegfs_opts[] =
     FFMPEGFS_OPT("--oldnamescheme=%u",              m_oldnamescheme, 0),
     FFMPEGFS_OPT("oldnamescheme=%u",                m_oldnamescheme, 0),
     // Experimental
-    FFMPEGFS_OPT("--win_smb_fix=%u",                m_win_smb_fix, 0),
-    FFMPEGFS_OPT("win_smb_fix=%u",                  m_win_smb_fix, 0),
+    FFMPEGFS_OPT("--win_smb_fix=%u",                m_win_smb_fix, 1),
+    FFMPEGFS_OPT("win_smb_fix=%u",                  m_win_smb_fix, 1),
     // FFmpegfs options
     FFMPEGFS_OPT("-d",                              m_debug, 1),
     FFMPEGFS_OPT("debug",                           m_debug, 1),
@@ -326,15 +316,17 @@ static struct fuse_opt ffmpegfs_opts[] =
     FUSE_OPT_KEY("--help",                          KEY_HELP),
     FUSE_OPT_KEY("-V",                              KEY_VERSION),
     FUSE_OPT_KEY("--version",                       KEY_VERSION),
+    FUSE_OPT_KEY("-c",                              KEY_FFMPEG_CAPS),
+    FUSE_OPT_KEY("--capabilities",                  KEY_FFMPEG_CAPS),
     FUSE_OPT_KEY("-d",                              KEY_KEEP_OPT),
     FUSE_OPT_KEY("debug",                           KEY_KEEP_OPT),
     FUSE_OPT_END
 };
 
 typedef std::map<std::string, AUTOCOPY, comp> AUTOCOPY_MAP;     /**< @brief Map command line option to AUTOCOPY enum */
-typedef std::map<std::string, PROFILE, comp> PROFILE_MAP;       /**< @brief Map command line option to PROFILE enum  */
-typedef std::map<std::string, PRORESLEVEL, comp> LEVEL_MAP;     /**< @brief Map command line option to LEVEL enum  */
-typedef std::map<std::string, RECODESAME, comp> RECODESAME_MAP; /**< @brief Map command line option to RECODESAME enum  */
+typedef std::map<std::string, PROFILE, comp> PROFILE_MAP;       /**< @brief Map command line option to PROFILE enum */
+typedef std::map<std::string, PRORESLEVEL, comp> LEVEL_MAP;     /**< @brief Map command line option to LEVEL enum */
+typedef std::map<std::string, RECODESAME, comp> RECODESAME_MAP; /**< @brief Map command line option to RECODESAME enum */
 
 /**
   * List of AUTOCOPY options
@@ -431,13 +423,13 @@ static void usage()
  * @brief Iterate through all elements in map and search for the passed element.
  * @param[in] mapOfWords - map to search.
  * @param[in] value - Search value
- * @return If found, retuns const_iterator to element. Returns mapOfWords.end() if not.
+ * @return If found, retuns const_iterator to element. Returns mapOfWords.cend() if not.
  */
 template <typename T>
 static typename std::map<std::string, T, comp>::const_iterator search_by_value(const std::map<std::string, T, comp> & mapOfWords, T value)
 {
-    typename std::map<std::string, T, comp>::const_iterator it = mapOfWords.begin();
-    while (it != mapOfWords.end())
+    typename std::map<std::string, T, comp>::const_iterator it = mapOfWords.cbegin();
+    while (it != mapOfWords.cend())
     {
         if (it->second == value)
         {
@@ -445,7 +437,7 @@ static typename std::map<std::string, T, comp>::const_iterator search_by_value(c
         }
         it++;
     }
-    return mapOfWords.end();
+    return mapOfWords.cend();
 }
 
 /**
@@ -470,7 +462,7 @@ static int get_bitrate(const std::string & arg, BITRATE *bitrate)
         int reti;
 
         // Check for decimal number
-        reti = compare(data, "^([1-9][0-9]*|0)?(bps)?$");
+        reti = reg_compare(data, "^([1-9][0-9]*|0)?(bps)?$", std::regex::icase);
 
         if (reti == -1)
         {
@@ -483,7 +475,7 @@ static int get_bitrate(const std::string & arg, BITRATE *bitrate)
         }
 
         // Check for number with optional descimal point and K modifier
-        reti = compare(data, "^[1-9][0-9]*(\\.[0-9]+)?K(bps)?$");
+        reti = reg_compare(data, "^[1-9][0-9]*(\\.[0-9]+)?K(bps)?$", std::regex::icase);
 
         if (reti == -1)
         {
@@ -496,7 +488,7 @@ static int get_bitrate(const std::string & arg, BITRATE *bitrate)
         }
 
         // Check for number with optional descimal point and M modifier
-        reti = compare(data, "^[1-9][0-9]*(\\.[0-9]+)?M(bps)?$");
+        reti = reg_compare(data, "^[1-9][0-9]*(\\.[0-9]+)?M(bps)?$", std::regex::icase);
 
         if (reti == -1)
         {
@@ -539,7 +531,7 @@ static int get_samplerate(const std::string & arg, int * samplerate)
         int reti;
 
         // Check for decimal number
-        reti = compare(data, "^([1-9][0-9]*|0)(Hz)?$");
+        reti = reg_compare(data, "^([1-9][0-9]*|0)(Hz)?$", std::regex::icase);
 
         if (reti == -1)
         {
@@ -552,7 +544,7 @@ static int get_samplerate(const std::string & arg, int * samplerate)
         }
 
         // Check for number with optional descimal point and K modifier
-        reti = compare(data, "^[1-9][0-9]*(\\.[0-9]+)?K(Hz)?$");
+        reti = reg_compare(data, "^[1-9][0-9]*(\\.[0-9]+)?K(Hz)?$", std::regex::icase);
 
         if (reti == -1)
         {
@@ -599,7 +591,7 @@ static int get_time(const std::string & arg, time_t *time)
         int reti;
 
         // Check for decimal number
-        reti = compare(data, "^([1-9][0-9]*|0)?s?$");
+        reti = reg_compare(data, "^([1-9][0-9]*|0)?s?$", std::regex::icase);
 
         if (reti == -1)
         {
@@ -612,7 +604,7 @@ static int get_time(const std::string & arg, time_t *time)
         }
 
         // Check for number with optional descimal point and m modifier
-        reti = compare(data, "^[1-9][0-9]*(\\.[0-9]+)?m$");
+        reti = reg_compare(data, "^[1-9][0-9]*(\\.[0-9]+)?m$", std::regex::icase);
 
         if (reti == -1)
         {
@@ -625,7 +617,7 @@ static int get_time(const std::string & arg, time_t *time)
         }
 
         // Check for number with optional descimal point and h modifier
-        reti = compare(data, "^[1-9][0-9]*(\\.[0-9]+)?h$");
+        reti = reg_compare(data, "^[1-9][0-9]*(\\.[0-9]+)?h$", std::regex::icase);
 
         if (reti == -1)
         {
@@ -638,7 +630,7 @@ static int get_time(const std::string & arg, time_t *time)
         }
 
         // Check for number with optional descimal point and d modifier
-        reti = compare(data, "^[1-9][0-9]*(\\.[0-9]+)?d$");
+        reti = reg_compare(data, "^[1-9][0-9]*(\\.[0-9]+)?d$", std::regex::icase);
 
         if (reti == -1)
         {
@@ -651,7 +643,7 @@ static int get_time(const std::string & arg, time_t *time)
         }
 
         // Check for number with optional descimal point and w modifier
-        reti = compare(data, "^[1-9][0-9]*(\\.[0-9]+)?w$");
+        reti = reg_compare(data, "^[1-9][0-9]*(\\.[0-9]+)?w$", std::regex::icase);
 
         if (reti == -1)
         {
@@ -697,7 +689,7 @@ static int get_size(const std::string & arg, size_t *size)
         int reti;
 
         // Check for decimal number
-        reti = compare(data, "^([1-9][0-9]*|0)?B?$");
+        reti = reg_compare(data, "^([1-9][0-9]*|0)?B?$", std::regex::icase);
 
         if (reti == -1)
         {
@@ -710,7 +702,7 @@ static int get_size(const std::string & arg, size_t *size)
         }
 
         // Check for number with optional descimal point and K/KB modifier
-        reti = compare(data, "^[1-9][0-9]*(\\.[0-9]+)?KB?$");
+        reti = reg_compare(data, "^[1-9][0-9]*(\\.[0-9]+)?KB?$", std::regex::icase);
 
         if (reti == -1)
         {
@@ -723,7 +715,7 @@ static int get_size(const std::string & arg, size_t *size)
         }
 
         // Check for number with optional descimal point and M/MB modifier
-        reti = compare(data, "^[1-9][0-9]*(\\.[0-9]+)?MB?$");
+        reti = reg_compare(data, "^[1-9][0-9]*(\\.[0-9]+)?MB?$", std::regex::icase);
 
         if (reti == -1)
         {
@@ -736,7 +728,7 @@ static int get_size(const std::string & arg, size_t *size)
         }
 
         // Check for number with optional descimal point and G/GB modifier
-        reti = compare(data, "^[1-9][0-9]*(\\.[0-9]+)?GB?$");
+        reti = reg_compare(data, "^[1-9][0-9]*(\\.[0-9]+)?GB?$", std::regex::icase);
 
         if (reti == -1)
         {
@@ -749,7 +741,7 @@ static int get_size(const std::string & arg, size_t *size)
         }
 
         // Check for number with optional descimal point and T/TB modifier
-        reti = compare(data, "^[1-9][0-9]*(\\.[0-9]+)?TB?$");
+        reti = reg_compare(data, "^[1-9][0-9]*(\\.[0-9]+)?TB?$", std::regex::icase);
 
         if (reti == -1)
         {
@@ -780,7 +772,6 @@ static int get_size(const std::string & arg, size_t *size)
  */
 static int get_desttype(const std::string & arg, FFmpegfs_Format format[2])
 {
-    /** @todo: evaluate this function */
     size_t pos = arg.find('=');
 
     if (pos != std::string::npos)
@@ -828,9 +819,9 @@ static int get_autocopy(const std::string & arg, AUTOCOPY *autocopy)
     {
         std::string data(arg.substr(pos + 1));
 
-        auto it = autocopy_map.find(data);
+        AUTOCOPY_MAP::const_iterator it = autocopy_map.find(data);
 
-        if (it == autocopy_map.end())
+        if (it == autocopy_map.cend())
         {
             std::fprintf(stderr, "INVALID PARAMETER: Invalid autocopy option: %s\n", data.c_str());
             return -1;
@@ -854,7 +845,7 @@ static int get_autocopy(const std::string & arg, AUTOCOPY *autocopy)
 static std::string get_autocopy_text(AUTOCOPY autocopy)
 {
     AUTOCOPY_MAP::const_iterator it = search_by_value(autocopy_map, autocopy);
-    if (it != autocopy_map.end())
+    if (it != autocopy_map.cend())
     {
         return it->first;
     }
@@ -875,9 +866,9 @@ static int get_recodesame(const std::string & arg, RECODESAME *recode)
     {
         std::string data(arg.substr(pos + 1));
 
-        auto it = recode_map.find(data);
+        RECODESAME_MAP::const_iterator it = recode_map.find(data);
 
-        if (it == recode_map.end())
+        if (it == recode_map.cend())
         {
             std::fprintf(stderr, "INVALID PARAMETER: Invalid recode option: %s\n", data.c_str());
             return -1;
@@ -901,7 +892,7 @@ static int get_recodesame(const std::string & arg, RECODESAME *recode)
 static std::string get_recodesame_text(RECODESAME recode)
 {
     RECODESAME_MAP::const_iterator it = search_by_value(recode_map, recode);
-    if (it != recode_map.end())
+    if (it != recode_map.cend())
     {
         return it->first;
     }
@@ -922,9 +913,9 @@ static int get_profile(const std::string & arg, PROFILE *profile)
     {
         std::string data(arg.substr(pos + 1));
 
-        auto it = profile_map.find(data);
+        PROFILE_MAP::const_iterator it = profile_map.find(data);
 
-        if (it == profile_map.end())
+        if (it == profile_map.cend())
         {
             std::fprintf(stderr, "INVALID PARAMETER: Invalid profile: %s\n", data.c_str());
             return -1;
@@ -948,7 +939,7 @@ static int get_profile(const std::string & arg, PROFILE *profile)
 static std::string get_profile_text(PROFILE profile)
 {
     PROFILE_MAP::const_iterator it = search_by_value(profile_map, profile);
-    if (it != profile_map.end())
+    if (it != profile_map.cend())
     {
         return it->first;
     }
@@ -970,9 +961,9 @@ static int get_level(const std::string & arg, PRORESLEVEL *level)
     {
         std::string data(arg.substr(pos + 1));
 
-        auto it = level_map.find(data);
+        LEVEL_MAP::const_iterator it = level_map.find(data);
 
-        if (it == level_map.end())
+        if (it == level_map.cend())
         {
             std::fprintf(stderr, "INVALID PARAMETER: Invalid level: %s\n", data.c_str());
             return -1;
@@ -997,7 +988,7 @@ static int get_level(const std::string & arg, PRORESLEVEL *level)
 static std::string get_level_text(PRORESLEVEL level)
 {
     LEVEL_MAP::const_iterator it = search_by_value(level_map, level);
-    if (it != level_map.end())
+    if (it != level_map.cend())
     {
         return it->first;
     }
@@ -1159,7 +1150,6 @@ static int ffmpegfs_opt_proc(void* data, const char* arg, int key, struct fuse_a
     }
     case KEY_VERSION:
     {
-        /** @todo: Also output this information in debug mode */
         std::printf("-------------------------------------------------------------------------------------------\n");
 
 #ifdef __GNUC__
@@ -1171,7 +1161,7 @@ static int ffmpegfs_opt_proc(void* data, const char* arg, int key, struct fuse_a
 #endif
         std::printf("%-20s: %s\n\n", "configuration", CONFIGURE_ARGS);
 
-        std::printf("%-20s: %s\n", PACKAGE_NAME " Version", PACKAGE_VERSION);
+        std::printf("%-20s: %s\n", PACKAGE_NAME " Version", FFMPEFS_VERSION);
 
         std::printf("%s", ffmpeg_libinfo().c_str());
 
@@ -1188,10 +1178,18 @@ static int ffmpegfs_opt_proc(void* data, const char* arg, int key, struct fuse_a
         fuse_opt_add_arg(outargs, "--version");
         fuse_main(outargs->argc, outargs->argv, &ffmpegfs_ops, nullptr);
 
+        exit(0);
+    }
+    case KEY_FFMPEG_CAPS:
+    {
         std::printf("-------------------------------------------------------------------------------------------\n\n");
-        std::printf("FFMpeg capabilities\n\n");
 
-        show_formats_devices(0);
+        std::printf("%-20s: %s\n", PACKAGE_NAME " Version", FFMPEFS_VERSION);
+        std::printf("%s", ffmpeg_libinfo().c_str());
+
+        std::printf("\nFFMpeg capabilities\n\n");
+
+        show_caps(0);
 
         exit(0);
     }
@@ -1281,7 +1279,18 @@ static int ffmpegfs_opt_proc(void* data, const char* arg, int key, struct fuse_a
     }
     case KEY_LOGFILE:
     {
-        return get_value(arg, &params.m_logfile);
+        std::string logfile;
+        int res = get_value(arg, &logfile);
+
+        if (res)
+        {
+            return res;
+        }
+
+        expand_path(&params.m_logfile, logfile);
+        sanitise_filepath(&params.m_logfile);
+
+        return 0;
     }
     }
 
@@ -1335,7 +1344,7 @@ static void print_params(void)
     Logging::trace(nullptr, "Codec             : %1", get_codec_name(params.m_format[0].video_codec_id(), true));
     Logging::trace(nullptr, "Bitrate           : %1", format_bitrate(params.m_videobitrate).c_str());
     Logging::trace(nullptr, "--------- HLS Options ---------");
-    Logging::trace(nullptr, "Segment Duration  : %1", format_time(params.m_segment_duration).c_str());
+    Logging::trace(nullptr, "Segment Duration  : %1", format_time(static_cast<time_t>(params.m_segment_duration / AV_TIME_BASE)).c_str());
     Logging::trace(nullptr, "--------- Virtual Script ---------");
     Logging::trace(nullptr, "Create script     : %1", params.m_enablescript ? "yes" : "no");
     Logging::trace(nullptr, "Script file name  : %1", params.m_scriptfile.c_str());
@@ -1363,7 +1372,7 @@ static void print_params(void)
     Logging::trace(nullptr, "Min. DVD Chapter  : %1", format_duration(params.m_min_dvd_chapter_duration * AV_TIME_BASE).c_str());
     Logging::trace(nullptr, "Old Name Scheme   : %1", params.m_oldnamescheme ? "yes" : "no");
     Logging::trace(nullptr, "--------- Experimental Options ---------");
-    Logging::trace(nullptr, "Windows 10 Fix    : %1", params.m_win_smb_fix ? "inactive" : "SMB Lockup Fix Active");
+    Logging::trace(nullptr, "Windows 10 Fix    : %1", params.m_win_smb_fix ? "SMB Lockup Fix Active" : "inactive");
 }
 
 /**
@@ -1382,10 +1391,10 @@ int main(int argc, char *argv[])
     // Check if run from other process group like mount and if so, inhibit startup message
     if (getppid() == getpgid(0))
     {
-        std::printf("%s V%s\n", PACKAGE_NAME, PACKAGE_VERSION);
+        std::printf("%s V%s\n", PACKAGE_NAME, FFMPEFS_VERSION);
         std::printf("Copyright (C) 2006-2008 David Collett\n"
                     "Copyright (C) 2008-2012 K. Henriksson\n"
-                    "Copyright (C) 2017-2020 FFmpeg support by Norbert Schlia (nschlia@oblivion-software.de)\n\n");
+                    "Copyright (C) 2017-2021 FFmpeg support by Norbert Schlia (nschlia@oblivion-software.de)\n\n");
     }
 
     init_fuse_ops();
