@@ -1,7 +1,7 @@
 /*
  * Copyright (C) 2006-2008 David Collett
  * Copyright (C) 2008-2013 K. Henriksson
- * Copyright (C) 2017-2020 FFmpeg support by Norbert Schlia (nschlia@oblivion-software.de)
+ * Copyright (C) 2017-2021 FFmpeg support by Norbert Schlia (nschlia@oblivion-software.de)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -34,7 +34,7 @@
  * @author Norbert Schlia (nschlia@oblivion-software.de)
  * @copyright Copyright (C) 2006-2008 David Collett @n
  * Copyright (C) 2008-2013 K. Henriksson @n
- * Copyright (C) 2017-2020 FFmpeg support by Norbert Schlia (nschlia@oblivion-software.de)
+ * Copyright (C) 2017-2021 FFmpeg support by Norbert Schlia (nschlia@oblivion-software.de)
  */
 
 /** @mainpage FFmpegfs FUSE Filesystem
@@ -245,28 +245,37 @@ bool            transcoder_cache_clear(void);
  * For Bluray/DVD/VCD actually no physical input file exists, so virtual file and origfile are the same.
  *
  * @param[in] type - Type of virtual file.
- * @param[in] virtfilepath - Name of virtual file.
+ * @param[in] virtfile - Name of virtual file.
  * @param[in] stbuf - stat buffer with file size, time etc.
  * @param[in] flags - One of the VIRTUALFLAG_* flags to control the detailed behaviour.
  * @return Returns constant pointer to VIRTUALFILE object of file, nullptr if not found
  */
-LPVIRTUALFILE   insert_file(VIRTUALTYPE type, const std::string & virtfilepath, const struct stat *stbuf, int flags = VIRTUALFLAG_NONE);
+LPVIRTUALFILE   insert_file(VIRTUALTYPE type, const std::string & virtfile, const struct stat *stbuf, int flags = VIRTUALFLAG_NONE);
 /**
  * @brief Add new virtual file to internal list. If the file already exists, it will be updated.
  * @param[in] type - Type of virtual file.
- * @param[in] virtfilepath - Name of virtual file.
+ * @param[in] virtfile - Name of virtual file.
  * @param[in] origfile - Original file name.
  * @param[in] stbuf - stat buffer with file size, time etc.
  * @param[in] flags - One of the VIRTUALFLAG_* flags to control the detailed behaviour.
  * @return Returns constant pointer to VIRTUALFILE object of file, nullptr if not found
  */
-LPVIRTUALFILE   insert_file(VIRTUALTYPE type, const std::string &virtfilepath, const std::string & origfile, const struct stat *stbuf, int flags = VIRTUALFLAG_NONE);
+LPVIRTUALFILE   insert_file(VIRTUALTYPE type, const std::string &virtfile, const std::string & origfile, const struct stat *stbuf, int flags = VIRTUALFLAG_NONE);
+/**
+ * @brief Add new virtual directory to internal list. If the file already exists, it will be updated.
+ * @param[in] type - Type of virtual file.
+ * @param[in] virtdir - Name of virtual directory.
+ * @param[in] stbuf - stat buffer with file size, time etc.
+ * @param[in] flags - One of the VIRTUALFLAG_* flags to control the detailed behaviour.
+ * @return Returns constant pointer to VIRTUALFILE object of file, nullptr if not found
+ */
+LPVIRTUALFILE   insert_dir(VIRTUALTYPE type, const std::string & virtdir, const struct stat * stbuf, int flags = VIRTUALFLAG_NONE);
 /**
  * @brief Find file in cache.
- * @param[in] virtfilepath - Virtual filename and path of file to find.
+ * @param[in] virtfile - Virtual filename and path of file to find.
  * @return If found, returns VIRTUALFILE object, if not found returns nullptr.
  */
-LPVIRTUALFILE   find_file(const std::string &virtfilepath);
+LPVIRTUALFILE   find_file(const std::string &virtfile);
 /**
  * @brief Check if path has already been parsed.
  * Only useful if for DVD, Bluray or VCD where it is guaranteed that all files have been parsed whenever the directory is in the hash.
@@ -301,3 +310,24 @@ LPVIRTUALFILE   find_original(std::string *filepath);
  * @return Returns contstant pointer to VIRTUALFILE object of file, nullptr if not found
  */
 LPVIRTUALFILE   find_parent(const std::string & origpath);
+
+/**
+ * @brief Wrapper to the Fuse filler function.
+ * @param buf[inout] - The buffer passed to the readdir() operation. May be nullptr.
+ * @param filler[in] - Function pointer to the Fuse update function.  May be nullptr.
+ * @param name[in] - The file name of the directory entry. Do not include the path!
+ * @param stat[in] - File attributes, can be nullptr.
+ * @param off[in] - Offset of the next entry or zero.
+ * @return 1 if buffer is full, zero otherwise or if buf or filler is nullptr.
+ */
+int             add_fuse_entry(void *buf, fuse_fill_dir_t filler, const char * name, const struct stat *stbuf, off_t off);
+
+/**
+ * @brief Create the ./.. entries for a virtual directory
+ * @param buf[inout] - The buffer passed to the readdir() operation. May be nullptr.
+ * @param filler[in] - Function pointer to the Fuse update function.  May be nullptr.
+ * @param stat[in] - File attributes, can be nullptr.
+ * @param off[in] - Offset of the next entry or zero.
+ * @return 1 if buffer is full, zero otherwise or if buf or filler is nullptr.
+ */
+int             add_dotdot(void *buf, fuse_fill_dir_t filler, const struct stat *stbuf, off_t off);
