@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2017-2020 by Norbert Schlia (nschlia@oblivion-software.de)
+ * Copyright (C) 2017-2021 by Norbert Schlia (nschlia@oblivion-software.de)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -29,7 +29,7 @@
  * @ingroup ffmpegfs
  *
  * @author Norbert Schlia (nschlia@oblivion-software.de)
- * @copyright Copyright (C) 2017-2020 Norbert Schlia (nschlia@oblivion-software.de)
+ * @copyright Copyright (C) 2017-2021 Norbert Schlia (nschlia@oblivion-software.de)
  */
 
 #ifndef FILEIO_H
@@ -47,19 +47,17 @@
 #include <map>
 
 // Disable annoying warnings outside our code
+#ifdef __cplusplus
+extern "C" {
+#endif
 #pragma GCC diagnostic push
 #pragma GCC diagnostic ignored "-Wconversion"
 #pragma GCC diagnostic ignored "-Wsign-conversion"
-#ifdef __cplusplus
-extern "C" {
-#endif
-extern "C" {
 #include <libavutil/avutil.h>
-}
+#pragma GCC diagnostic pop
 #ifdef __cplusplus
 }
 #endif
-#pragma GCC diagnostic pop
 
 #pragma pack(push, 1)
 
@@ -128,7 +126,7 @@ typedef struct VIRTUALFILE
 
     }
 
-    uint32_t get_segment_count() const;
+    uint32_t get_segment_count() const;                             /**< @brief Number of HLS segments in set */
 
     VIRTUALTYPE         m_type;                                     /**< @brief Type of this virtual file */
     int                 m_flags;                                    /**< @brief One of the VIRTUALFLAG_* flags */
