@@ -1,7 +1,7 @@
 /*
  * Copyright (C) 2006-2008 David Collett
  * Copyright (C) 2008-2012 K. Henriksson
- * Copyright (C) 2017-2020 FFmpeg support by Norbert Schlia (nschlia@oblivion-software.de)
+ * Copyright (C) 2017-2021 FFmpeg support by Norbert Schlia (nschlia@oblivion-software.de)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -30,7 +30,7 @@
  * @author Norbert Schlia (nschlia@oblivion-software.de)
  * @copyright Copyright (C) 2006-2008 David Collett @n
  * Copyright (C) 2008-2013 K. Henriksson @n
- * Copyright (C) 2017-2020 FFmpeg support by Norbert Schlia (nschlia@oblivion-software.de)
+ * Copyright (C) 2017-2021 FFmpeg support by Norbert Schlia (nschlia@oblivion-software.de)
  */
 
 #include "transcode.h"
@@ -70,6 +70,7 @@ static void             translate_path(std::string *origpath, const char* path);
 static bool             transcoded_name(std::string *filepath, FFmpegfs_Format **current_format = nullptr);
 static filenamemap::const_iterator find_prefix(const filenamemap & map, const std::string & search_for);
 static int              get_source_properties(const std::string & origpath, LPVIRTUALFILE virtualfile);
+static int              make_hls_fileset(void * buf, fuse_fill_dir_t filler, const std::string & origpath, LPVIRTUALFILE virtualfile);
 
 static int              ffmpegfs_readlink(const char *path, char *buf, size_t size);
 static int              ffmpegfs_readdir(const char *path, void *buf, fuse_fill_dir_t filler, off_t offset, struct fuse_file_info *fi);
@@ -95,7 +96,7 @@ thread_pool*                tp;                 /**< @brief Thread pool object *
 
 /**
   *
-  * List of passthrough file extension
+  * List of passthrough file extensions
   *
   * Note: Must be in ascending aplhabetical order for the
   * binary_search function used on it to work! New elements
@@ -165,6 +166,7 @@ static const std::list<std::string> passthrough_map =
     "JXR",		// JPEG-XR/Microsoft HD Photo format
     "KDC",		// Kodak digital camera format
     "LBM",		// Interchange File Format
+    "M3U8",     // Apple HTTP Live Streaming
     "MIFF",
     "MNG",		// Multiple Network Graphics
     "MRC",		// MRC format
@@ -268,12 +270,7 @@ static void init_stat(struct stat * stbuf, size_t fsize, time_t ftime, bool dire
         stbuf->st_nlink = 1;
     }
 
-#if defined __x86_64__ || !defined __USE_FILE_OFFSET64
-    stbuf->st_size = static_cast<__off_t>(fsize);
-#else
-    stbuf->st_size = static_cast<__off64_t>(fsize);
-#endif
-    stbuf->st_blocks = (stbuf->st_size + 512 - 1) / 512;
+    stat_set_size(stbuf, fsize);
 
     // Set current user as owner
     stbuf->st_uid = getuid();
@@ -301,9 +298,9 @@ static LPVIRTUALFILE make_file(void *buf, fuse_fill_dir_t filler, VIRTUALTYPE ty
 
     init_stat(&stbuf, fsize, ftime, false);
 
-    if (buf != nullptr && filler != nullptr)
+    if (add_fuse_entry(buf, filler, filename.c_str(), &stbuf, 0))
     {
-        filler(buf, filename.c_str(), &stbuf, 0);
+        return nullptr;
     }
 
     return insert_file(type, origpath + filename, &stbuf, flags);
@@ -384,7 +381,7 @@ static bool transcoded_name(std::string * filepath, FFmpegfs_Format **current_fo
     {
         std::string ext;
 
-        if (!find_ext(&ext, *filepath) || !std::binary_search(passthrough_map.begin(), passthrough_map.end(), ext, nocasecompare))
+        if (!find_ext(&ext, *filepath) || !std::binary_search(passthrough_map.cbegin(), passthrough_map.cend(), ext, nocasecompare))
         {
             FFmpegfs_Format *ffmpegfs_format = params.current_format(*filepath);
 
@@ -418,12 +415,12 @@ static bool transcoded_name(std::string * filepath, FFmpegfs_Format **current_fo
  * @brief Find mapped file by prefix. Normally used to find a path.
  * @param[in] map - File map with virtual files.
  * @param[in] search_for - Prefix (path) to search for.
- * @return If found, returns const_iterator to map entry. Returns map.end() if not found.
+ * @return If found, returns const_iterator to map entry. Returns map.cend() if not found.
  */
 static filenamemap::const_iterator find_prefix(const filenamemap & map, const std::string & search_for)
 {
     filenamemap::const_iterator it = map.lower_bound(search_for);
-    if (it != map.end())
+    if (it != map.cend())
     {
         const std::string & key = it->first;
         if (key.compare(0, search_for.size(), search_for) == 0) // Really a prefix?
@@ -431,21 +428,22 @@ static filenamemap::const_iterator find_prefix(const filenamemap & map, const st
             return it;
         }
     }
-    return map.end();
+    return map.cend();
 }
 
-LPVIRTUALFILE insert_file(VIRTUALTYPE type, const std::string & virtfilepath, const struct stat * stbuf, int flags)
+LPVIRTUALFILE insert_file(VIRTUALTYPE type, const std::string & virtfile, const struct stat * stbuf, int flags)
 {
-    return insert_file(type, virtfilepath, virtfilepath, stbuf, flags);
+    return insert_file(type, virtfile, virtfile, stbuf, flags);
 }
 
-LPVIRTUALFILE insert_file(VIRTUALTYPE type, const std::string & virtfilepath, const std::string & origfile, const struct stat * stbuf, int flags)
+LPVIRTUALFILE insert_file(VIRTUALTYPE type, const std::string & _virtfile, const std::string & _origfile, const struct stat * stbuf, int flags)
 {
-    std::string sanitised_filepath = sanitise_filepath(virtfilepath);
+    std::string virtfile(sanitise_filepath(_virtfile));
+    std::string origfile(sanitise_filepath(_origfile));
 
-    filenamemap::iterator it    = filenames.find(sanitised_filepath);
+    filenamemap::iterator it    = filenames.find(virtfile);
 
-    if (it != filenames.end())
+    if (it != filenames.cend())
     {
         VIRTUALFILE & virtualfile = it->second;
 
@@ -454,7 +452,7 @@ LPVIRTUALFILE insert_file(VIRTUALTYPE type, const std::string & virtfilepath, co
         virtualfile.m_type          = type;
         virtualfile.m_flags         = flags;
         virtualfile.m_format_idx    = params.guess_format_idx(origfile);
-        virtualfile.m_origfile      = sanitise_filepath(origfile);
+        virtualfile.m_origfile      = origfile;
     }
     else
     {
@@ -465,18 +463,48 @@ LPVIRTUALFILE insert_file(VIRTUALTYPE type, const std::string & virtfilepath, co
         virtualfile.m_type          = type;
         virtualfile.m_flags         = flags;
         virtualfile.m_format_idx    = params.guess_format_idx(origfile);
-        virtualfile.m_origfile      = sanitise_filepath(origfile);
+        virtualfile.m_origfile      = origfile;
 
-        filenames.insert(make_pair(sanitised_filepath, virtualfile));
-        it    = filenames.find(sanitised_filepath);
+        filenames.insert(make_pair(virtfile, virtualfile));
+        it    = filenames.find(virtfile);
     }
 
     return &it->second;
 }
 
-LPVIRTUALFILE find_file(const std::string & virtfilepath)
+LPVIRTUALFILE insert_dir(VIRTUALTYPE type, const std::string & virtdir, const struct stat * stbuf, int flags)
 {
-    filenamemap::iterator it = filenames.find(sanitise_filepath(virtfilepath));
+    struct stat stbufdir;
+
+    flags |= VIRTUALFLAG_DIRECTORY | VIRTUALFLAG_FILESET;
+
+    std::memcpy(&stbufdir, stbuf, sizeof(stbufdir));
+
+    // Change file to directory for the frame set
+    // Change file to virtual directory for the frame set. Keep permissions.
+    stbufdir.st_mode  &= ~static_cast<mode_t>(S_IFREG | S_IFLNK);
+    stbufdir.st_mode  |= S_IFDIR;
+    stbufdir.st_nlink = 2;
+    stbufdir.st_size  = stbufdir.st_blksize;
+
+    std::string path(virtdir);
+    append_sep(&path);
+
+    if (params.m_format[0].is_frameset())
+    {
+        flags |= VIRTUALFLAG_FRAME;
+    }
+    else if (params.m_format[0].is_hls())
+    {
+        flags |= VIRTUALFLAG_HLS;
+    }
+
+    return insert_file(type, path, &stbufdir, flags);
+}
+
+LPVIRTUALFILE find_file(const std::string & virtfile)
+{
+    filenamemap::iterator it = filenames.find(sanitise_filepath(virtfile));
 
     errno = 0;
 
@@ -491,19 +519,24 @@ bool check_path(const std::string & path)
 {
     filenamemap::const_iterator it = find_prefix(filenames, path);
 
-    return (it != filenames.end());
+    return (it != filenames.cend());
 }
 
 int load_path(const std::string & path, const struct stat *statbuf, void *buf, fuse_fill_dir_t filler)
 {
+    if (buf == nullptr)
+    {
+        // We can't add anything here if buf == nullptr
+        return 0;
+    }
     int title_count = 0;
 
     filenamemap::const_iterator it = filenames.lower_bound(path);
-    while (it != filenames.end())
+    while (it != filenames.cend())
     {
-        std::string key = it->first;
-        remove_filename(&key);
-        if (key == path) // Really a prefix?
+        std::string virtfilepath = it->first;
+        remove_filename(&virtfilepath);
+        if (virtfilepath == path) // Really a prefix?
         {
             LPCVIRTUALFILE virtualfile = &it->second;
             struct stat stbuf;
@@ -516,10 +549,9 @@ int load_path(const std::string & path, const struct stat *statbuf, void *buf, f
 
             memcpy(&stbuf, statbuf, sizeof(struct stat));
 
-            stbuf.st_size   = virtualfile->m_st.st_size;
-            stbuf.st_blocks = (stbuf.st_size + 512 - 1) / 512;
+            stat_set_size(&stbuf, static_cast<size_t>(virtualfile->m_st.st_size));
 
-            if (buf != nullptr && filler(buf, destfile.c_str(), &stbuf, 0))
+            if (add_fuse_entry(buf, filler, destfile.c_str(), &stbuf, 0))
             {
                 // break;
             }
@@ -577,14 +609,14 @@ LPVIRTUALFILE find_original(std::string * filepath)
         {
             std::string dir(*filepath);
             std::string filename(*filepath);
-            std::string tmppath;
+            std::string origfile;
             struct dirent **namelist;
             struct stat stbuf;
             int count;
             int found = 0;
 
             remove_filename(&dir);
-            tmppath = dir;
+            origfile = dir;
 
             count = scandir(dir.c_str(), &namelist, selector, nullptr);
             if (count == -1)
@@ -603,36 +635,36 @@ LPVIRTUALFILE find_original(std::string * filepath)
 
             for (int n = 0; n < count; n++)
             {
-                if (!found && !compare(namelist[n]->d_name, filename))
+                if (!found && !reg_compare(namelist[n]->d_name, filename, std::regex::icase))
                 {
-                    append_filename(&tmppath, namelist[n]->d_name);
+                    append_filename(&origfile, namelist[n]->d_name);
                     found = 1;
                 }
                 free(namelist[n]);
             }
             free(namelist);
 
-            sanitise_filepath(&tmppath);
+            sanitise_filepath(&origfile);
 
-            if (found && lstat(tmppath.c_str(), &stbuf) == 0)
+            if (found && lstat(origfile.c_str(), &stbuf) == 0)
             {
                 // File exists with this extension
                 LPVIRTUALFILE virtualfile;
 
-                if (*filepath != tmppath)
+                if (*filepath != origfile)
                 {
-                    virtualfile = insert_file(VIRTUALTYPE_DISK, *filepath, tmppath, &stbuf);
-                    *filepath = tmppath;
+                    virtualfile = insert_file(VIRTUALTYPE_DISK, *filepath, origfile, &stbuf); ///<* @todo This probably won't work, need to redo "Fallback to old method"
+                    *filepath = origfile;
                 }
                 else
                 {
-                    virtualfile = insert_file(VIRTUALTYPE_DISK, tmppath, &stbuf, VIRTUALFLAG_PASSTHROUGH); /**< @todo Feature #2447 / Issue #25: add command line option */
+                    virtualfile = insert_file(VIRTUALTYPE_DISK, origfile, &stbuf, VIRTUALFLAG_PASSTHROUGH);
                 }
                 return virtualfile;
             }
             else
             {
-                // File does not exist; not an error
+                // File does not exist; this is a virtual file, not an error
                 errno = 0;
             }
         }
@@ -707,7 +739,15 @@ static int get_source_properties(const std::string & origpath, LPVIRTUALFILE vir
     return 0;
 }
 
-int make_hls(void * buf, fuse_fill_dir_t filler, const std::string & origpath, LPVIRTUALFILE virtualfile)
+/**
+ * @brief Build a virtual HLS file set
+ * @param[in, out] buf - The buffer passed to the readdir() operation.
+ * @param[in, out] filler - Function to add an entry in a readdir() operation (see https://libfuse.github.io/doxygen/fuse_8h.html#a7dd132de66a5cc2add2a4eff5d435660)
+ * @param[in] origpath - The original file
+ * @param[in] virtualfile - LPCVIRTUALFILE of file to create file set for
+ * @return On success, returns 0. On error, returns -errno.
+ */
+static int make_hls_fileset(void * buf, fuse_fill_dir_t filler, const std::string & origpath, LPVIRTUALFILE virtualfile)
 {
     // Generate set of TS segment files and necessary M3U lists
     LPVIRTUALFILE child_file;
@@ -751,15 +791,14 @@ int make_hls(void * buf, fuse_fill_dir_t filler, const std::string & origpath, L
                 "#EXT-X-MEDIA-SEQUENCE:1\n";
 
         int64_t remaining_duration  = virtualfile->m_duration % params.m_segment_duration;
-        size_t  segment_size        = virtualfile->m_predicted_size / virtualfile->get_segment_count(); // @todo: Feature #2506 - calculate correct file size
-        size_t  remaining_size      = virtualfile->m_predicted_size % virtualfile->get_segment_count(); // @todo: Feature #2506 - calculate correct file size
+        size_t  segment_size        = virtualfile->m_predicted_size / virtualfile->get_segment_count();
+        size_t  remaining_size      = virtualfile->m_predicted_size % virtualfile->get_segment_count();
 
         for (uint32_t file_no = 1; file_no <= virtualfile->get_segment_count(); file_no++)
         {
             std::string buffer;
             std::string segment_name = make_filename(file_no, params.current_format(virtualfile)->fileext());
 
-            //**< @todo; Rework this test code
             struct stat stbuf;
             std::string cachefile;
             std::string _origpath(origpath);
@@ -793,15 +832,7 @@ int make_hls(void * buf, fuse_fill_dir_t filler, const std::string & origpath, L
             }
 
             index_0_av_contents += buffer;
-#if 0			
-			std::string buffer2(origpath);
-			
-			//remove_filename(&buffer2);
-			buffer2 = replace_all(buffer2, "/mnt/sdc1/", "https://mp3mania.de/files/hls/1M/");
-            index_0_av_contents += buffer2 + "/" + segment_name;
-#else		
             index_0_av_contents += segment_name;
-#endif
             index_0_av_contents += "\n";
         }
 
@@ -814,7 +845,7 @@ int make_hls(void * buf, fuse_fill_dir_t filler, const std::string & origpath, L
         std::copy(index_0_av_contents.begin(), index_0_av_contents.end(), std::back_inserter(child_file->m_file_contents));
 
         {
-            //**< @todo; Rework this test code
+            // Demo code adapted from: https://github.com/video-dev/hls.js/
             std::string hls_html;
 
             hls_html =
@@ -822,7 +853,7 @@ int make_hls(void * buf, fuse_fill_dir_t filler, const std::string & origpath, L
                     "\n"
                     "<head>\n"
                     "    <title>HLS Demo</title>\n"
-                    "    <script src=\"https://cdn.jsdelivr.net/npm/hls.js@0.12.2/dist/hls.min.js\"></script>\n"
+                    "    <script src=\"https://cdn.jsdelivr.net/npm/hls.js@latest\"></script>\n"
                     "    <meta charset=\"utf-8\">\n"
                     "</head>\n"
                     "\n"
@@ -832,24 +863,27 @@ int make_hls(void * buf, fuse_fill_dir_t filler, const std::string & origpath, L
                     "        <video height=\"600\" id=\"video\" controls></video>\n"
                     "    </center>\n"
                     "    <script>\n"
-                    "        if (Hls.isSupported()) {\n"
-                    "            var video = document.getElementById(\"video\");\n"
-                    "            var hls = new Hls();\n"
-                    "            hls.loadSource(\"index_0_av.m3u8\");\n"
-                    "            hls.attachMedia(video);\n"
-                    "            hls.on(Hls.Events.MANIFEST_PARSED, function() {\n"
-                    "                video.play();\n"
-                    "            });\n"
-                    "        }\n"
-                    "        // hls.js is not supported on platforms that do not have Media Source Extensions (MSE) enabled.\n"
-                    "        // When the browser has built-in HLS support (check using `canPlayType`), we can provide an HLS manifest (i.e. .m3u8 URL) directly to the video element throught the `src` property.\n"
-                    "        // This is using the built-in support of the plain video element, without using hls.js.\n"
-                    "        else if (video.canPlayType(\"application/vnd.apple.mpegurl\")) {\n"
-                    "            video.src = \"index_0_av.m3u8\";\n"
-                    "            video.addEventListener(\"canplay\", function() {\n"
-                    "                video.play();\n"
-                    "            });\n"
-                    "        }\n"
+                    "      var video = document.getElementById(\"video\");\n"
+                    "      var videoSrc = \"index_0_av.m3u8\";\n"
+                    "      if (Hls.isSupported()) {\n"
+                    "        var hls = new Hls();\n"
+                    "        hls.loadSource(videoSrc);\n"
+                    "        hls.attachMedia(video);\n"
+                    "        hls.on(Hls.Events.MANIFEST_PARSED, function() {\n"
+                    "          video.play();\n"
+                    "        });\n"
+                    "      }\n"
+                    "      // hls.js is not supported on platforms that do not have Media Source Extensions (MSE) enabled.\n"
+                    "      // When the browser has built-in HLS support (check using `canPlayType`), we can provide an HLS manifest (i.e. .m3u8 URL) directly to the video element through the `src` property.\n"
+                    "      // This is using the built-in support of the plain video element, without using hls.js.\n"
+                    "      // Note: it would be more normal to wait on the 'canplay' event below however on Safari (where you are most likely to find built-in HLS support) the video.src URL must be on the user-driven\n"
+                    "      // white-list before a 'canplay' event will be emitted; the last video event that can be reliably listened-for when the URL is not on the white-list is 'loadedmetadata'.\n"
+                    "      else if (video.canPlayType(\"application/vnd.apple.mpegurl\")) {\n"
+                    "        video.src = videoSrc;\n"
+                    "        video.addEventListener(\"loadedmetadata\", function() {\n"
+                    "          video.play();\n"
+                    "        });\n"
+                    "    }\n"
                     "    </script>\n"
                     "</body>\n"
                     "\n"
@@ -987,7 +1021,7 @@ static int ffmpegfs_readdir(const char *path, void *buf, fuse_fill_dir_t filler,
                         }
                     }
 
-                    if (filler(buf, filename.c_str(), &stbuf, 0))
+                    if (add_fuse_entry(buf, filler, filename.c_str(), &stbuf, 0))
                     {
                         break;
                     }
@@ -1005,6 +1039,11 @@ static int ffmpegfs_readdir(const char *path, void *buf, fuse_fill_dir_t filler,
     }
     else
     {
+        if (virtualfile->m_flags & (VIRTUALFLAG_FILESET | VIRTUALFLAG_FRAME | VIRTUALFLAG_HLS | VIRTUALFLAG_DIRECTORY))
+        {
+            add_dotdot(buf, filler, &virtualfile->m_st, 0);
+        }
+
         FFmpegfs_Format *ffmpegfs_format = params.current_format(virtualfile);
 
         if (ffmpegfs_format->is_frameset())
@@ -1023,12 +1062,12 @@ static int ffmpegfs_readdir(const char *path, void *buf, fuse_fill_dir_t filler,
 
             for (uint32_t frame_no = 1; frame_no <= virtualfile->m_video_frame_count; frame_no++)
             {
-                make_file(buf, filler, virtualfile->m_type, origpath, make_filename(frame_no, params.current_format(virtualfile)->fileext()), virtualfile->m_predicted_size, virtualfile->m_st.st_ctime, VIRTUALFLAG_FRAME); /**< @todo Calculate correct file size for frame image */
+                make_file(buf, filler, virtualfile->m_type, origpath, make_filename(frame_no, params.current_format(virtualfile)->fileext()), virtualfile->m_predicted_size, virtualfile->m_st.st_ctime, VIRTUALFLAG_FRAME); /**< @todo Calculate correct file size for frame image in set */
             }
         }
         else if (ffmpegfs_format->is_hls())
         {
-            int res = make_hls(buf, filler, origpath, virtualfile);
+            int res = make_hls_fileset(buf, filler, origpath, virtualfile);
             if (res < 0)
             {
                 return res;
@@ -1071,7 +1110,6 @@ static int ffmpegfs_getattr(const char *path, struct stat *stbuf)
             if (!current_format->is_multiformat())
             {
                 // Pass-through for regular files
-                /**< @todo Feature #2447 / Issue #25:  Reencode to same file format as source file */
                 Logging::trace(origpath, "getattr: Not transcoding existing file.");
                 errno = 0;
                 return 0;
@@ -1102,8 +1140,8 @@ static int ffmpegfs_getattr(const char *path, struct stat *stbuf)
     }
     else if (flags & VIRTUALFLAG_PASSTHROUGH && lstat(origpath.c_str(), stbuf) == 0)
     {
-        // File pysically exists and is marked as pass-through
-        Logging::trace(origpath, "getattr: File is marked as passthrough.");
+        // File physically exists and is marked as passthrough
+        Logging::debug(origpath, "getattr: File not recoded because --recodesame=NO.");
         errno = 0;
         return 0;
     }
@@ -1142,7 +1180,7 @@ static int ffmpegfs_getattr(const char *path, struct stat *stbuf)
     }
     case VIRTUALTYPE_DISK:
     {
-        if (virtualfile != nullptr && (flags & (VIRTUALFLAG_FRAME | VIRTUALFLAG_DIRECTORY | VIRTUALFLAG_HLS)))
+        if (virtualfile != nullptr && (flags & (VIRTUALFLAG_FRAME | VIRTUALFLAG_HLS | VIRTUALFLAG_DIRECTORY)))
         {
             mempcpy(stbuf, &virtualfile->m_st, sizeof(struct stat));
 
@@ -1159,43 +1197,43 @@ static int ffmpegfs_getattr(const char *path, struct stat *stbuf)
 #if defined(USE_LIBBLURAY) || defined(USE_LIBDVD) || defined(USE_LIBVCD)
                 int res = 0;
 
-                std::string _origpath(origpath);
-
-                remove_filename(&_origpath);
-
                 virtualfile = find_original(&origpath);
 
                 if (virtualfile == nullptr)
                 {
+                    std::string pathonly(origpath);
+                    remove_filename(&pathonly);
+
 #ifdef USE_LIBVCD
                     if (res <= 0)
                     {
                         // Returns -errno or number or titles on VCD
-                        res = check_vcd(_origpath);
+                        res = check_vcd(pathonly);
                     }
 #endif // USE_LIBVCD
 #ifdef USE_LIBDVD
                     if (res <= 0)
                     {
                         // Returns -errno or number or titles on DVD
-                        res = check_dvd(_origpath);
+                        res = check_dvd(pathonly);
                     }
 #endif // USE_LIBDVD
 #ifdef USE_LIBBLURAY
                     if (res <= 0)
                     {
                         // Returns -errno or number or titles on Bluray
-                        res = check_bluray(_origpath);
+                        res = check_bluray(pathonly);
                     }
 #endif // USE_LIBBLURAY
 
-                    if (params.m_format[0].is_hls())
+                    if (params.m_format[0].is_frameset())
                     {
                         LPVIRTUALFILE parent_file = find_parent(origpath);
 
                         if (parent_file != nullptr && (parent_file->m_flags & VIRTUALFLAG_DIRECTORY) && (parent_file->m_flags & VIRTUALFLAG_FILESET))
                         {
-                            if (!parent_file->m_video_frame_count)  //***< @todo: was ist mit audio only? Werden die dann nicht immer wieder gecheckt?!?
+                            // Generate set of all frames
+                            if (!parent_file->m_video_frame_count)
                             {
                                 int res = get_source_properties(origpath, parent_file);
                                 if (res < 0)
@@ -1204,7 +1242,42 @@ static int ffmpegfs_getattr(const char *path, struct stat *stbuf)
                                 }
                             }
 
-                            make_hls(nullptr, nullptr, parent_file->m_origfile + "/", parent_file); // TEST
+                            for (uint32_t frame_no = 1; frame_no <= parent_file->m_video_frame_count; frame_no++)
+                            {
+                                make_file(nullptr, nullptr, parent_file->m_type, parent_file->m_origfile + "/", make_filename(frame_no, params.current_format(parent_file)->fileext()), parent_file->m_predicted_size, parent_file->m_st.st_ctime, VIRTUALFLAG_FRAME); /**< @todo Calculate correct file size for frame image in set */
+                            }
+
+                            LPVIRTUALFILE virtualfile2 = find_original(origpath);
+                            if (virtualfile2 == nullptr)
+                            {
+                                // File does not exist
+                                return -ENOENT;
+                            }
+
+                            mempcpy(stbuf, &virtualfile2->m_st, sizeof(struct stat));
+
+                            // Clear errors
+                            errno = 0;
+
+                            return 0;
+                        }
+                    }
+                    else if (params.m_format[0].is_hls())
+                    {
+                        LPVIRTUALFILE parent_file = find_parent(origpath);
+
+                        if (parent_file != nullptr && (parent_file->m_flags & VIRTUALFLAG_DIRECTORY) && (parent_file->m_flags & VIRTUALFLAG_FILESET))
+                        {
+                            if (!parent_file->m_video_frame_count)  //***< @todo HLS format: Do audio files source properties get checked over and over?
+                            {
+                                int res = get_source_properties(origpath, parent_file);
+                                if (res < 0)
+                                {
+                                    return res;
+                                }
+                            }
+
+                            make_hls_fileset(nullptr, nullptr, parent_file->m_origfile + "/", parent_file); // TEST
 
                             LPVIRTUALFILE virtualfile2 = find_original(origpath);
                             if (virtualfile2 == nullptr)
@@ -1285,12 +1358,7 @@ static int ffmpegfs_getattr(const char *path, struct stat *stbuf)
                         return -errno;
                     }
 
-#if defined __x86_64__ || !defined __USE_FILE_OFFSET64
-                    stbuf->st_size = static_cast<__off_t>(transcoder_get_size(cache_entry));
-#else
-                    stbuf->st_size = static_cast<__off64_t>(transcoder_get_size(cache_entry));
-#endif
-                    stbuf->st_blocks = (stbuf->st_size + 512 - 1) / 512;
+                    stat_set_size(stbuf, transcoder_get_size(cache_entry));
 
                     transcoder_delete(cache_entry);
                 }
@@ -1338,7 +1406,7 @@ static int ffmpegfs_fgetattr(const char *path, struct stat * stbuf, struct fuse_
 
     if ((virtualfile == nullptr || virtualfile->m_flags & VIRTUALFLAG_PASSTHROUGH) && lstat(origpath.c_str(), stbuf) == 0)
     {
-        // pass-through for regular files
+        // passthrough for regular files
         errno = 0;
         return 0;
     }
@@ -1400,12 +1468,7 @@ static int ffmpegfs_fgetattr(const char *path, struct stat * stbuf, struct fuse_
 
                 uint32_t segment_no = 0;
 
-#if defined __x86_64__ || !defined __USE_FILE_OFFSET64
-                stbuf->st_size = static_cast<__off_t>(transcoder_buffer_watermark(cache_entry, segment_no));
-#else
-                stbuf->st_size = static_cast<__off64_t>(transcoder_buffer_watermark(cache_entry, segment_no));
-#endif
-                stbuf->st_blocks = (stbuf->st_size + 512 - 1) / 512;
+                stat_set_size(stbuf, transcoder_buffer_watermark(cache_entry, segment_no));
             }
         }
 
@@ -1652,7 +1715,7 @@ static int ffmpegfs_read(const char *path, char *buf, size_t size, off_t _offset
             if (!frame_no)
             {
                 errno = EINVAL;
-                Logging::error(origpath.c_str(), "read: Unable to deduct frame no. from file name (%1): (%2) %3", filename, errno, strerror(errno));
+                Logging::error(origpath.c_str(), "read: Unable to deduct frame no. from file name (%1): (%2) %3", filename.c_str(), errno, strerror(errno));
                 return -errno;
             }
 
@@ -1679,7 +1742,7 @@ static int ffmpegfs_read(const char *path, char *buf, size_t size, off_t _offset
                 if (!segment_no)
                 {
                     errno = EINVAL;
-                    Logging::error(origpath.c_str(), "read: Unable to deduct segment no. from file name (%1): (%2) %3", filename, errno, strerror(errno));
+                    Logging::error(origpath.c_str(), "read: Unable to deduct segment no. from file name (%1): (%2) %3", filename.c_str(), errno, strerror(errno));
                     return -errno;
                 }
             }
@@ -1720,7 +1783,7 @@ static int ffmpegfs_statfs(const char *path, struct statvfs *stbuf)
 
     translate_path(&origpath, path);
 
-    // pass-through for regular files
+    // passthrough for regular files
     if (statvfs(origpath.c_str(), stbuf) == 0)
     {
         return -errno;
@@ -1762,7 +1825,7 @@ static int ffmpegfs_release(const char *path, struct fuse_file_info *fi)
             if (!segment_no)
             {
                 errno = EINVAL;
-                Logging::error(path, "read: Unable to deduct segment no. from file name (%1): (%2) %3", filename, errno, strerror(errno));
+                Logging::error(path, "read: Unable to deduct segment no. from file name (%1): (%2) %3", filename.c_str(), errno, strerror(errno));
             }
             else
             {
@@ -1807,7 +1870,7 @@ static void sighandler(int signum)
  */
 static void *ffmpegfs_init(struct fuse_conn_info *conn)
 {
-    Logging::info(nullptr, "%1 V%2 initialising.", PACKAGE_NAME, PACKAGE_VERSION);
+    Logging::info(nullptr, "%1 V%2 initialising.", PACKAGE_NAME, FFMPEFS_VERSION);
     Logging::info(nullptr, "Mapping '%1' to '%2'.", params.m_basepath.c_str(), params.m_mountpath.c_str());
 
     struct sigaction sa;
@@ -1852,8 +1915,8 @@ static void *ffmpegfs_init(struct fuse_conn_info *conn)
  */
 static void ffmpegfs_destroy(__attribute__((unused)) void * p)
 {
-    Logging::info(nullptr, "%1 V%2 terminating", PACKAGE_NAME, PACKAGE_VERSION);
-    std::printf("%s V%s terminating\n", PACKAGE_NAME, PACKAGE_VERSION);
+    Logging::info(nullptr, "%1 V%2 terminating", PACKAGE_NAME, FFMPEFS_VERSION);
+    std::printf("%s V%s terminating\n", PACKAGE_NAME, FFMPEFS_VERSION);
 
     stop_cache_maintenance();
 
@@ -1869,7 +1932,7 @@ static void ffmpegfs_destroy(__attribute__((unused)) void * p)
 
     script_file.clear();
 
-    Logging::info(nullptr, "%1 V%2 terminated", PACKAGE_NAME, PACKAGE_VERSION);
+    Logging::info(nullptr, "%1 V%2 terminated", PACKAGE_NAME, FFMPEFS_VERSION);
 }
 
 /**
@@ -1890,3 +1953,29 @@ static std::string get_number(const char *path, uint32_t *value)
     return filename;
 }
 
+int add_fuse_entry(void *buf, fuse_fill_dir_t filler, const char * name, const struct stat *stbuf, off_t off)
+{
+    if (buf == nullptr || filler == nullptr)
+    {
+        return 0;
+    }
+
+    return filler(buf, name, stbuf, off);
+}
+
+int add_dotdot(void *buf, fuse_fill_dir_t filler, const struct stat *stbuf, off_t off)
+{
+    struct stat *stbuf2 = nullptr;
+    struct stat stbuf3;
+
+    if (stbuf != nullptr)
+    {
+        stbuf2 = &stbuf3;
+        init_stat(stbuf2, 0, stbuf->st_ctime, true);
+    }
+
+    add_fuse_entry(buf, filler, ".", stbuf2, off);
+    add_fuse_entry(buf, filler, "..", stbuf2, off);
+
+    return 0;
+}
