@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2017 Original author K. Henriksson @n
- * Copyright (C) 2017-2020 Norbert Schlia (nschlia@oblivion-software.de)
+ * Copyright (C) 2017-2021 Norbert Schlia (nschlia@oblivion-software.de)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -28,7 +28,7 @@
  *
  * @author K. Henriksson, Norbert Schlia (nschlia@oblivion-software.de)
  * @copyright Copyright (C) 2017 Original author K. Henriksson @n
- * Copyright (C) 2017-2020 Norbert Schlia (nschlia@oblivion-software.de)
+ * Copyright (C) 2017-2021 Norbert Schlia (nschlia@oblivion-software.de)
  */
 
 #ifndef LOGGING_H
@@ -122,7 +122,7 @@ private:
         /**
          * @brief Construct Logger object
          */
-        Logger() :
+        explicit Logger() :
             m_loglevel(level::DEBUG) {}
         /**
          * @brief Destroy Logger object
@@ -279,16 +279,17 @@ protected:
     /**
      * @brief Standard format_helper without parameters
      * @param[in] string_to_update - original string
+     * @param size - unused
      * @return Original string
      */
     static std::string format_helper(
             const std::string &string_to_update,
-            const size_t);
+            const size_t __attribute__((unused)) size);
 
     /**
      * @brief format_helper with variadic parameters
      *
-     * Calls itself recurisively until all tokens are replaced.
+     * Calls itself recursively until all tokens are replaced.
      *
      * @param[in] string_to_search - format string to be searched
      * @param[in] index_to_replace - index numer (%n) to be replaced. May be present 0...x times.
@@ -371,10 +372,10 @@ protected:
     const bool      m_to_syslog;                    /**< @brief Whether to write log output to syslog. */
 };
 
-constexpr auto ERROR    = Logging::level::ERROR;    /**< @brief Shorthand for log level ERROR */
-constexpr auto WARNING  = Logging::level::WARNING;  /**< @brief Shorthand for log level WARNING */
-constexpr auto INFO     = Logging::level::INFO;     /**< @brief Shorthand for log level INFO */
-constexpr auto DEBUG    = Logging::level::DEBUG;    /**< @brief Shorthand for log level DEBUG */
-constexpr auto TRACE    = Logging::level::TRACE;    /**< @brief Shorthand for log level TRACE */
+constexpr Logging::level ERROR    = Logging::level::ERROR;    /**< @brief Shorthand for log level ERROR */
+constexpr Logging::level WARNING  = Logging::level::WARNING;  /**< @brief Shorthand for log level WARNING */
+constexpr Logging::level INFO     = Logging::level::INFO;     /**< @brief Shorthand for log level INFO */
+constexpr Logging::level DEBUG    = Logging::level::DEBUG;    /**< @brief Shorthand for log level DEBUG */
+constexpr Logging::level TRACE    = Logging::level::TRACE;    /**< @brief Shorthand for log level TRACE */
 
 #endif
