@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2017-2020 Norbert Schlia (nschlia@oblivion-software.de)
+ * Copyright (C) 2017-2021 Norbert Schlia (nschlia@oblivion-software.de)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -31,7 +31,7 @@
  * @ingroup ffmpegfs
  *
  * @author Norbert Schlia (nschlia@oblivion-software.de)
- * @copyright Copyright (C) 2017-2020 Norbert Schlia (nschlia@oblivion-software.de)
+ * @copyright Copyright (C) 2017-2021 Norbert Schlia (nschlia@oblivion-software.de)
  */
 
 #include "vcdio.h"
@@ -52,7 +52,7 @@ VcdIO::VcdIO()
 
 VcdIO::~VcdIO()
 {
-    close();
+    _close();
 }
 
 VIRTUALTYPE VcdIO::type() const
@@ -124,7 +124,7 @@ int VcdIO::error() const
 
 int64_t VcdIO::duration() const
 {
-    return AV_NOPTS_VALUE;  /** @todo: implement me */
+    return AV_NOPTS_VALUE;
 }
 
 size_t VcdIO::size() const
@@ -197,6 +197,11 @@ bool VcdIO::eof() const
 }
 
 void VcdIO::close()
+{
+    _close();
+}
+
+void VcdIO::_close()
 {
     FILE *fpi = m_fpi;
     if (fpi != nullptr)
